window.ngui(() => {
    return {
        emit: [
            'update:purposes',
        ],
        props: [
            'modelValue',
            'purposeOptions',
            'selectedAlgo'
        ],   
        data() {
            return {
                localPurposeOptions: [],
                renderBins: false,
                algoRequiringBinning : [
                    {
                        'method':'thresholdsGrouping',
                        'model': '',
                    }
                ],
            }
        },
        methods: {
            toggleBinsVisibility(extInfos) {
                let renderBins = false;
                const needsBinning = this.algoRequiringBinning.filter((algo) => {
                    return (this.selectedAlgo.method == algo.method && this.selectedAlgo.model == algo.model);
                });

                if (
                    needsBinning.length > 0 &&
                    this.ctx.intelli.hasContinuousValue(extInfos) &&
                    this.modelValue.purposes.includes('computeThresholds')
                ) {  
                    renderBins = true;
                }
                return renderBins;
            }
        },
        watch: {
            'modelValue.extInfos'(newExtInfos) {
                this.renderBins = this.toggleBinsVisibility(newExtInfos);
            },                     
          },        
        mounted() {
            // Set default value
            this.localPurposeOptions = this.purposeOptions;

            // if create set all options as default
            if (!this.modelValue.pmModel.path && this.purposeOptions) {
                this.modelValue.purposes = this.purposeOptions;
            }

            this.renderBins = this.toggleBinsVisibility(this.modelValue.extInfos);
        },
        nodes() {
            let purposes = [];

            this.localPurposeOptions.forEach((option) => {
                purposes.push(
                    { 
                        type: 'NgFormInput',
                        props: {
                            type: 'checkbox',
                            val: option,
                            label: option,
                            col: 12,
                            modelValue: this.modelValue.purposes,
                            'onUpdate:modelValue': (value) => {
                                this.modelValue.purposes = value;
                                this.renderBins = this.toggleBinsVisibility(this.modelValue.extInfos);
                                this.$emit('update:purposes', value);
                            },
                            rules: [(val) => (val == undefined | val.length !== 0) || this.$t("ruleNotEmpty")]                                                                       
                        },
                    },
                );
            });    
            if (this.renderBins) {      
                purposes = purposes.concat([
                    { 
                        type: 'NgFormInput',
                        props: {
                            type: 'number',
                            label: 'baseNumber',
                            col: 6,
                            modelValue: this.modelValue.bins.base,
                            'onUpdate:modelValue': (value) => {
                                this.modelValue.bins.base = value;
                            },
                            step: 'any',
                            rules: [
                                (val) => (val !== '' && val !== undefined) || this.$t('ruleNotEmpty'), 
                                (val) => val >= 0 || this.$t('ruleAbsolute'), 
                            ]
                        }
                    },                    
                    { 
                        type: 'NgFormInput',
                        props: {
                            type: 'number',
                            label: 'step',
                            col: 6,
                            modelValue: this.modelValue.bins.step,
                            'onUpdate:modelValue': (value) => {
                                this.modelValue.bins.step = value;
                            },
                            step: 'any',
                            rules: [
                                (val) => (val !== '' && val !== undefined) || this.$t('ruleNotEmpty'),
                                (val) => val > 0 || this.$t('ruleMinValue', {value: 1}),  // Value 0 throws np.arange error
                            ]
                        } 
                    }                   
                ]);
            }
            return purposes;                                                            
         }        
    }
});