
window.ngui(() => {
    return {
        props: [
            'job', 
        ],        
        data() {            
            return {
                testEnvironment: {
                    uuid: '',
                    name: '',
                    description: '',
                    widgets: [],       
                    timeAware: {
                        createdOn: '',
                        createdBy: '',
                        modifiedBy: '',
                        modifiedOn: ''                        
                    }
                },
                pageUuid: '',
                pageOptions: [],
                selectedDependentVarUuid: '',
                filteredResults: [],
                dependentOptions: [],
                results: {},
                jobConfiguration: {
                    nodeName: '',
                },
                explanatoryDataColumns: [
                    { name: 'name', i18nLabel: 'Name', field: (row) => {
                        return row.extInfos.name || '[N/A]';
                    }, sortable: true },
                    { name: 'path', i18nLabel: 'path', field: (row) => row.pmModel.path, sortable: true },
                    { name: 'interpolation', i18nLabel: 'interpolation', field: (row) => this.$t(row.interpolation.template), sortable: true },
                    { name: 'mandatory', i18nLabel: 'mandatory', field: (row) => (row.mandatory ? this.$t('yes') : this.$t('no')), sortable: true },
                ],
                resultColumns: [
                    { name: 'method', i18nLabel: 'method', field: (row) => this.$t(row.config.task.method), sortable: true },
                    { name: 'model', i18nLabel: 'model', field: (row) => row.config.task.model, sortable: true },
                    { name: 'explanatoryVariables', i18nLabel: 'explanatoryVariables', field: (row) => row.config.explanatoryVariables, sortable: true },   
                    { name: 'stats', i18nLabel: 'statistic', field: (row) => row.config.stats ?  this.$t(row.config.stats) : '-', sortable: true },
                    { name: 'metric', i18nLabel: 'metric', field: (row) => row.config.metric, sortable: true },
                    // { name: 'hyperParamTuningStart', i18nLabel: 'hyperParamTuningStart', sortable: false },                    
                    // { name: 'hyperParamTuningMetric', i18nLabel: 'hyperParamTuningMetric', field: (row) => { 
                    //     return row.config.gridSearch ?  row.config.gridSearch.tunedMetric.toFixed(3) : '[n/a]';
                    // }, sortable: true, align: 'center'  },
                    { name: 'visualize', i18nLabel: 'visualizationApply', field: 'visualize'},
                ],
                dialog: {
                    visualizeResult: false,
                    testEnvironment: false,
                },
                chartDataPayload: {
                    scriptFile: '',
                    scriptType: '',
                    scriptArguments: {
                        jobUuid:'',
                        resultUuid: '',
                    },
                    trainingDates: [],
                    measureDates: [],
                    histData: {
                        start: '',
                        end: '',
                        interval: undefined,                        
                    },
                },
                loading: false,
                selectedResult: {},
                updateTunedMetric: 0,
                socketClient: undefined,
                activeExplanatoryVariables: [],
            }
        },
        methods: {
            createTestEnvironment() {
                this.testEnvironment.uuid = this.ctx.uuidv4();
                this.testEnvironment.timeAware.createdOn = this.ctx.moment.getLocalizedNow().format();
                this.testEnvironment.timeAware.createdBy = this.ctx.httpClient.getUser()?.userName;
        
                this.ctx.filedb
                .insert(`app/BAI/pages/${this.testEnvironment.uuid}.yaml`, this.testEnvironment)
                .then(() => {
                    this.getPageOptions();
                    this.pageUuid = this.testEnvironment.uuid;
                })
                .catch((err) => {
                    this.ctx.log.error(err);
                })
                .finally(() => {
                    this.dialog.testEnvironment = false;
                });    
            },                                
            getActiveExplNodesByDependentNode() {
                let independent_nodes = [];

                // because multiple dependent variable can be set for one job
                const featuresSets = this.job.config.importantFeaturesSets.filter((set) => {
                    return (set.dependentVar.uuid === this.selectedDependentVarUuid);
                });
                
                if (featuresSets.length > 0) {
                    const active_independent_vars = featuresSets[0].independentVars.filter((independent_var) => {
                        return (independent_var.active)
                    });
    
                    active_independent_vars.forEach((independent_var) => {
                        const independen_node = this.job.config.nodes.filter((node) => {
                            return (independent_var.pmModel.path === node.pmModel.path && independent_var.pmModel.instanceUuid === node.pmModel.instanceUuid)
                        });
                        independent_nodes = independent_nodes.concat(independen_node);
                    });
                }

                this.activeExplanatoryVariables = independent_nodes;
            },
            getRandomColorHex() {
                return `#${Math.floor(Math.random() * 16777215).toString(16)}`;
            },            
            getPageOptions() {
                this.ctx.filedb
                .read('app/BAI/pages?pattern=*.yaml')
                .then((result) => {
                    const data = result.data;
                    if (data) { // no page might exist yet
                        this.pageOptions = data.map((item) => {
                            const page = item.data;
                            const label = `${page.name} ${page.description}`;
                            return { label, value: page.uuid };
                        });
                    }
                })
                .catch((err) => {
                    this.ctx.log.error(err);
                });                
            },
            async applyConfig() {
                this.loading = true;

                let gridSearch = {
                    model: '',
                    bestParams: '',
                    bestParamsEncoded: '',
                    bestScore: null,
                    tunedMetric: null
                }
                if (this.selectedResult.config.gridSearch) {
                    gridSearch  = { ...this.selectedResult.config.gridSearch,  model: this.selectedResult.config.task.model};
                }

                const intelliNodes = [];
                const configModelNodes = [];
                
                const dependentVariables = [];
                let independentVariables = [];

                const dependentJobNode = this.job.config.nodes.filter((node) => {
                    const dependentResultNode = this.selectedResult.config.dependentVar;                    
                    return (node.pmModel.processUuid === dependentResultNode.pmModel.processUuid && node.pmModel.path === dependentResultNode.pmModel.path);
                })[0];

                const dependentChartOptions =  { ...dependentJobNode.chartOptions, y: { opposite: false, ownAxis: true }};

                // create configModel node
                const configModelDependentNode = this.ctx.intelli.initNode();
                configModelDependentNode.uuid = this.ctx.uuidv4();
                configModelDependentNode.bins = dependentJobNode.bins;
                configModelDependentNode.pmModel = dependentJobNode.pmModel;
                configModelDependentNode.chartOptions = dependentChartOptions;
                configModelDependentNode.extInfos = dependentJobNode.extInfos;
                configModelDependentNode.interpolation = dependentJobNode.interpolation;
                configModelNodes.push(configModelDependentNode);
                dependentVariables.push(this.ctx.intelli.getNodeUniqueIdentifier(configModelDependentNode));

                // create intelli api payload
                intelliNodes.push({      
                    uuid: configModelDependentNode.uuid,              
                    dependentVariable: true,                                    
                    interpolation: dependentJobNode.interpolation,
                    pmModel : dependentJobNode.pmModel,
                    chartOptions: dependentChartOptions,
                    extInfos: dependentJobNode.extInfos,
                    bins: dependentJobNode.bins  
                });                
   
                // Get independent variables which are not nodes
                const independentDateTimeVars = this.selectedResult.config.explanatoryVariables.filter((explVar) => {
                    return (explVar.pmModel.path === '');
                }).map((independentNode) => {
                    return independentNode.name;
                });
                independentVariables = independentVariables.concat(independentDateTimeVars);

                // Get independent variables which are nodes
                const independentNodeVars = this.selectedResult.config.explanatoryVariables.filter((explVar) => {
                    return (explVar.pmModel.path !== '');
                });

                independentNodeVars.forEach((independentNodeVar) => {
                    const independentNode = this.job.config.nodes.filter((jobNode) => {
                        return (jobNode.pmModel.processUuid === independentNodeVar.pmModel.processUuid && jobNode.pmModel.path === independentNodeVar.pmModel.path);
                    })[0];

                    const independentChartOptions =  { ...independentNode.chartOptions, y: { opposite: true, ownAxis: true }};

                    // create configModel
                    const configModelIndependentNode = this.ctx.intelli.initNode();
                    configModelIndependentNode.uuid = this.ctx.uuidv4();
                    configModelIndependentNode.bins = independentNode.bins;
                    configModelIndependentNode.pmModel = independentNode.pmModel;
                    configModelIndependentNode.chartOptions = independentChartOptions;
                    configModelIndependentNode.extInfos = independentNode.extInfos;
                    configModelIndependentNode.interpolation = independentNode.interpolation;
                    configModelIndependentNode.purposes = ['computeThresholds']
                    configModelNodes.push(configModelIndependentNode);
                    independentVariables.push(this.ctx.intelli.getNodeUniqueIdentifier(configModelIndependentNode));

                    // create intelli api payload
                    intelliNodes.push({
                        uuid: configModelIndependentNode.uuid,
                        dependentVariable: false,                                    
                        interpolation: independentNode.interpolation,
                        pmModel : independentNode.pmModel,
                        chartOptions: independentChartOptions,
                        extInfos: independentNode.extInfos,
                        bins: independentNode.bins                                                              
                    });                    
                });

                const configModel = {
                    uuid: this.ctx.uuidv4(),                  
                    task: {
                        name: 'maxpermissiblevalues',
                        algorithm: {
                            method: this.selectedResult.config.task.method,
                            model: this.selectedResult.config.task.model,
                        },
                        metric: null,
                    },
                    interval: this.job.config.interval,
                    trainMethod: this.job.config.trainMethod,
                    trainingDates: this.job.config.trainingDates,
                    measureDates: this.job.config.trainMethod == 'isolatedWindows' ? this.job.config.measureDates : [],
                    trainTime: this.job.config.trainTime,
                    nodes: configModelNodes,
                    dependentVariables,
                    independentVariables,
                    explanatoryDateTimeVariables: independentDateTimeVars,
                    holidayLocation: this.job.config.holidayLocation,
                    gridSearch,
                    minElementsPerBin: 3,
                    trespassing: {
                        upper: true,
                        lower: true,
                    },
                    periodsDay: {
                        train: this.job.config.periodsDay.train,
                        retrain: this.job.config.periodsDay.retrain,
                        ignoreEvents: true,
                    },
                    sensitivityThreshold: {
                        upper: 1,
                        lower: 1,
                    },
                    eventCount: {
                        upper: 3,
                        lower: 3,
                    },
                    minErrorDistance: {
                        upper: 0,
                        lower: 0,
                    },
                    thresholdColor: '#99ff99',
                    expectedValueColor: '#47b547',
                };

                
                const nodeOptions = intelliNodes.map((node) => {
                    return {
                        name: this.ctx.intelli.getNodeChartName(node),
                        type: 'line',
                        data: {
                            yUuidRef: node.uuid,
                        },
                        yAxis: this.ctx.intelli.getNodeChartName(node),
                    };
                });

                const widget = this.ctx.intelli.initWidget();
                widget.uuid = this.ctx.uuidv4();
                widget.environment = 'test';
                widget.name = this.ctx.utils.getName(dependentJobNode) || dependentJobNode.pmModel.path;
                widget.configModelUuid = configModel.uuid;
                widget.outputUuid = this.ctx.uuidv4();
                widget.type = 'maxpermissiblevalues';
                widget.timeAware.createdAt = this.ctx.moment.getLocalizedNow().format();
                widget.timeAware.createdBy = this.ctx.httpClient.getUser()?.userName;            

                try {                
                    const pagePath = `app/BAI/pages/${this.pageUuid}.yaml`;
                    const response = await this.ctx.filedb.read(pagePath);
                    const page = response.data.data;
                    page.widgets.push(widget.uuid);            
    
                    await this.ctx.filedb.insert(`app/BAI/configModel/${configModel.uuid}.json`, configModel);
                    await this.ctx.filedb.insert(`app/BAI/widgets/${widget.uuid}.yaml`, widget);
                    await this.ctx.filedb.insert(pagePath, page);
                } catch (err) {
                    this.ctx.log.error(err);
                }

                const trainingDates = configModel.trainingDates;
                const measureDates = configModel.measureDates;

                const payload = {
                    scriptType: 'maxpermissiblevalues',
                    scriptFile: 'compute_thresholds.py',
                    scriptArguments: {
                        configModelUuid: configModel.uuid,
                        trainingDates,
                        measureDates,
                        dependentNodeName: this.ctx.intelli.getNodeChartName(dependentJobNode)
                    },
                    outputUuid: widget.outputUuid,
                    dates: trainingDates.concat(measureDates),
                    holidayLocation: configModel.holidayLocation,
                    nodes: intelliNodes,
                    nodeOptions,
                    histData: {
                        start: trainingDates[trainingDates.length - 1].start,
                        end: this.ctx.moment.getLocalizedNow().format(),
                        interval: configModel.interval,
                    }
                }

                this.ctx.intelli.chartData(payload)
                    .then(() => {
                        const sandboxRoute = this.ctx.routing.router.resolve({
                            name: 'sandboxWidgetDetail',
                            query: {
                                page: this.pageUuid,
                                widget: widget.uuid,
                            },
                        });
                        window.open(sandboxRoute.href, '_self');
                    })
                    .catch((err) => {
                        this.ctx.log.error(err);
                    })
                    .finally(() => {
                        this.loading = false;
                    });                
            },            
            getDefaultDates(interval = undefined) {
                let months;
                switch (interval) {
                  case -1:
                    months = 12;
                    break;
                  case -2:
                    months = 12 * 3;
                    break;
                  default:
                    months = 3;
                }
            
                const start = this.ctx.moment.getLocalizedNow().subtract(months, 'month').format();
                const end = this.ctx.moment.getLocalizedNow().format();
            
                return { start, end };
            },            
            onVisualizeResult(obj) {
                this.selectedResult = obj.row;

                this.chartDataPayload.scriptFile = 'metrics/visualize.py';
                this.chartDataPayload.scriptType = 'maxpermissiblevalues';
     
                this.chartDataPayload.dates = this.job.config.trainingDates.concat(this.job.config.measureDates);
                // Set dependent node first
                this.chartDataPayload.nodes = this.job.config.nodes.filter((node) => node.uuid === this.selectedResult.config.dependentVar.uuid);
                // Add independent nodes
                this.selectedResult.config.explanatoryVariables.forEach((explNode) => {
                    const independen_node = this.job.config.nodes.filter((node) => {
                        return (explNode.pmModel.path === node.pmModel.path && explNode.pmModel.instanceUuid === node.pmModel.instanceUuid)
                    });
                    this.chartDataPayload.nodes = this.chartDataPayload.nodes.concat(independen_node);
                });

                // Set color code to dependent node
                this.chartDataPayload.nodes = this.chartDataPayload.nodes.map((node) => {
                    node.chartOptions.color = '#C10015';
                    if (!node.dependentVariable) {
                        node.chartOptions.color = this.getRandomColorHex();
                        node.chartOptions.y =  { opposite: true};                    
                    }
                    return node;
                });

                this.chartDataPayload.nodeOptions = this.chartDataPayload.nodes.map((node) => {
                    return {
                        name: this.ctx.intelli.getNodeChartName(node),
                        type: 'line',
                        data: {
                          yUuidRef: node.uuid,
                        },
                        yAxis: this.ctx.intelli.getNodeChartName(node),
                      }
                });
          
                const dates = this.getDefaultDates(this.job.config.interval);
                const histData = {
                  start: dates.start,
                  end: dates.end,
                  interval: this.job.config.interval,
                };
          
 

                const dependentNode = this.job.config.nodes?.filter((node) => {
                    return (node.dependentVariable)
                })[0];

                this.chartDataPayload.scriptArguments = { 
                    jobUuid: this.job.uuid, 
                    resultUuid: this.selectedResult.uuid, 
                    trainingDates: this.job.config.trainingDates,
                    measureDates: this.job.config.measureDates,
                    dependentNodeName: this.ctx.intelli.getNodeChartName(dependentNode)                    
                };
                this.chartDataPayload.histData = histData;
                this.chartDataPayload.holidayLocation = this.job.config.holidayLocation;
                this.chartDataPayload.outputUuid = this.selectedResult.uuid;
                this.dialog.visualizeResult = true;
            },
            getTuningJob(mainJob, result) {
                return new Promise((resolve, reject) => {
                    if (result.config.tuningJobUuid !== '') {
                        this.ctx.filedb
                            .read(`app/BAI/jobs/${result.config.tuningJobUuid}.yaml`)
                            .then((response) => {
                                resolve(response.data.data);
                            })
                            .catch((err) => {
                                reject(err)
                            });
                    } else {
                        // Create hyperparameter tuning job
                        const tuningJob = this.ctx.intelli.initJob();
                        tuningJob.interpreter = 'filedb/scripts/maxpermissiblevalues/venv/Scripts/python';
                        tuningJob.type = 'hyperparametertuning';
                        tuningJob.script = 'filedb/scripts/maxpermissiblevalues/hyperparameter_tuning.py';
                        tuningJob.config.mainJobConf = mainJob.config;
                        tuningJob.config.resultUuid = result.uuid;
                        tuningJob.config.resultConf = result.config;
                        tuningJob.timeAware.createdAt = this.ctx.moment.getLocalizedNow().format();
                        tuningJob.timeAware.createdBy = this.ctx.httpClient.getUser()?.userName;


                        // Create inputTimeSeries (Data passed over disk)
                        const nodes = tuningJob.config.mainJobConf.nodes.map((node) => {
                            return {
                                dependentVariable: node.dependentVariable,
                                pmModel: node.pmModel,                      
                                interpolation: node.interpolation,
                                bins: node.bins,
                            }    
                        });

                        tuningJob.inputTimeSeries = {
                            nodes,
                            dates: tuningJob.config.mainJobConf.trainingDates.concat(tuningJob.config.mainJobConf.measureDates),
                            interval: tuningJob.config.mainJobConf.interval,
                            holidayLocation: tuningJob.config.mainJobConf.holidayLocation
                        }

                        this.ctx.filedb
                            .insert(`app/BAI/jobs/${tuningJob.uuid}.yaml`, tuningJob)
                            .then(() => {
                                resolve(tuningJob);
                            })
                            .catch((err) => {
                                reject(err);
                        });
                    }
                });

            },
            onHyperparamterTuning(mainJob, resultUuid) {
                if (this.socketClient.connected === false) {
                    this.connectWebSocket();
                }
                return new Promise((resolve, reject) => {
                    const results = mainJob.results.filter((result) => (result.uuid === resultUuid));
                    if (results.length === 0) {
                        reject('mainJob result not found');
                    }
                    const result = results[0];
                    let tuningJobUuid = '';
                    
                    this.getTuningJob(mainJob, result)
                    .then((tuningJob) => {
                        tuningJobUuid = tuningJob.uuid
                        // Store tuningJob to mainjob result to keep link
                        result.config.tuningJobUuid = tuningJobUuid;
                        return this.ctx.filedb.insert(`app/BAI/jobs/${mainJob.uuid}.yaml`, mainJob);
                    })
                    .then(() => {                
                        this.socketClient.send(JSON.stringify({ topic: tuningJobUuid }));
                        const url = this.ctx.endpoints.getIntelliService('/jobs/start');
                        return this.ctx.httpClient.post(url, {
                            jobUuid: tuningJobUuid
                        });
                    }).then(() => {
                        resolve(true);
                    }).catch((err) => {
                        reject(err);
                    });                     
                });            
            },            
            getFullNodeName(extInfos) {
                const name = extInfos?.name;
                const unit = extInfos?.unit;
                return extInfos?.unit ? `${name} ${unit}` : name;
            },
            getCard(title, value) {
                return [{
                    type: 'NgCard',
                    props: {
                    },
                    children:{
                        default: (slot) => { 
                            return [
                                {
                                    type: 'NgCardSection',
                                    props: {},
                                    children:{
                                        default: () => {
                                            return [
                                                {
                                                    type: 'NgText',
                                                    props: {
                                                        type: 'subtitle2',
                                                        text: this.$t(title),
                                                    },
                                                    
                                                }                                                                            
                                            ];
                                        }                        
                                    }                                                                                                       
                                },
                                {
                                    type: 'NgSeparator',
                                    props: {
                                        inset: true,
                                    },                                                                    
                                },
                                {
                                    type: 'NgCardSection',
                                    props: {},
                                    children:{
                                        default: () => {
                                            let rv = value;
                                            if (!Array.isArray(value)) {
                                                rv = [value];
                                            }  
                                            return rv;                       
                                        }                        
                                    }                                                                                                       
                                }
                            ]
                        }
                    }                                     
                }];                                    
            },
            createNode(author) {
                const node = this.getDefaultNode();
                node.uuid = this.ctx.uuidv4();
                node.timeAware.createdAt = this.ctx.moment.getLocalizedNow().format();
                node.timeAware.createdBy = author;
                return node;
            },
            getDefaultNode() {
                return {
                  uuid: '',
                  unit: '', // for custom unit
                  extInfos: {
                    name: '',
                    unit: '',
                    accType: '',
                  },
                  task: {
                    name: '',
                    method: '',
                  },
                  interpolation: {
                    template: 'rawValues',
                  },
                  configModelUuid: '',
                  description: '',
                  pmModel: {
                    processManagerUuid: '',
                    processUuid: '',
                    instanceUuid: '',
                    path: '',
                  },
                  chartOptions: {
                    y: {
                      ownAxis: false,
                      opposite: false,
                    },
                    color: '#c10015',
                    decimal: 0,
                  },
                  interval: undefined,
                  type: '',
                  actions: [],
                  timeAware: {
                    createdAt: '',
                    createdBy: '',
                    modifiedAt: '',
                    modifiedBy: '',
                  },
                };
              }, 
              getDependentVarsOptions(nodes) {
                const dependentVars = nodes.filter((node)  => {
                    return (node.dependentVariable === true);
                })
                return dependentVars.map((dependentVar) => {
                    return {'label': `${dependentVar.extInfos.name} ${dependentVar.pmModel.path}`, 'value': dependentVar.uuid};
                })                        
            },
            filterResultsByDependentVariable(results, selectedDependentVarUuid) {
                return results.filter((result) => {
                    return (result.config.dependentVar.uuid === selectedDependentVarUuid);
                });
            },            
            translateExplanatoryVariables(explVars) {
                return explVars.map((ev) => {
                    const timeVars = [
                        'hod',
                        'dow',
                        'woy',
                        'month',
                        'quarter',
                        'Mo2Sa-So',
                        'Mo2Fr-Sa-So',
                        'Mo2Fr-Sa2So',
                        'holiday'
                    ];
                    let explVar = timeVars.includes(ev.name) ? this.$t(ev.name) : ev.name;
                    
                    if (explVar === '' && ev.pmModel){
                        explVar = ev.pmModel.path;
                    }

                    if (ev.interpolationTemplate) {
                        explVar = `${explVar} (${this.$t(ev.interpolationTemplate)})`;
                    }
                    return explVar;                                                 
                });                
            },   
            resetTunedJobUuid() {
                // Delete tuningJob files
                const filteredResults = this.job.results.filter((result) => {
                    return (result.config.tuningJobUuid !== '');
                }); 
                filteredResults.forEach((result) => {
                    this.ctx.filedb
                    .remove(`app/BAI/jobs/${result.config.tuningJobUuid}.yaml`)
                    .catch(() => {});  
                });
    
                // Remove tuning job uuid from results
                this.job.results = this.job.results.map((result) => {
                    result.config.tuningJobUuid = '';
                    return result;
                });
                this.ctx.filedb
                    .insert(`app/BAI/jobs/${this.job.uuid}.yaml`, this.job)
                    .catch((err) => {
                        this.ctx.log.error(err);
                });          
            },
            connectWebSocket() {
                this.socketClient = new this.ctx.Socket();
                this.socketClient.setAuth(true);
                const url = this.ctx.endpoints.getIntelliService('/jobs/subscribe', '', true);            
                this.socketClient.connect(url);

                this.socketClient.onMessage((response) => {
                    if (response.data.error !== '') {
                        this.ctx.log.error(response.data.error);
                        this.resetTunedJobUuid();
                    } else {                
    
                        const result = this.job.results.filter((result) => {
                            return (result.uuid === response.data.data.resultUuid);
                        })[0];
                        result.config.gridSearch = response.data.data.gridSearch;
                        result.config.tuningJobUuid = '';               
                        this.ctx.filedb
                            .insert(`app/BAI/jobs/${this.job.uuid}.yaml`, this.job)
                            .catch((err) => {
                                this.ctx.log.error(err);
                        });
                    }
                });
                
                this.socketClient.onError((ev) => {                
                    this.ctx.log.error(ev);
                    this.resetTunedJobUuid();
                });          
                
                this.socketClient.onClosed((ev) => {
                    this.ctx.log.error(ev);
                    this.resetTunedJobUuid();
                });
            },
        },
        beforeMount() {
            this.connectWebSocket();
            if (this.job.results) {
                this.job.results.forEach((result) => {
                    if (result.config.tuningJobUuid) {
                        this.socketClient.send(JSON.stringify({ topic: result.config.tuningJobUuid }));                
                    }
                });
            }

            this.getPageOptions();
            this.jobConfiguration.nodeName = this.job.label;
            this.jobConfiguration.timeResolution = this.$t(this.ctx.intelli.getIntervalName(this.job.config.interval));
            this.jobConfiguration.trainMethod = this.$t(this.job.config.trainMethod);
            this.jobConfiguration.periodsDayTrain = this.job.config.periodsDay.train;
            this.jobConfiguration.periodsDayRetrain = this.job.config.periodsDay.retrain;
            this.jobConfiguration.trainingDates = this.job.config.trainingDates;
            this.jobConfiguration.measureDates = this.job.config.measureDates;
            this.jobConfiguration.trainTime = this.job.config.trainTime;

            this.dependentOptions = this.getDependentVarsOptions(this.job.config.nodes);
            this.selectedDependentVarUuid = this.dependentOptions[0].value;
            this.filteredResults = this.filterResultsByDependentVariable(this.job.results, this.selectedDependentVarUuid);
            this.getActiveExplNodesByDependentNode();
        },
        nodes() {
            const vNodes = [              
                {
                    type: 'NgSpacing',
                    props: {
                        my: 'md',
                    },
                    children: {
                        default: (slot) => {
                            return [
                                {
                                    type: 'NgText',
                                    props: {
                                        type: 'h6',
                                        text: this.jobConfiguration.nodeName,
                                        backgroundColor: 'grey',
                                        look: 'center'                                
                                    }
                                }                            
                            ];
                        }
                    }
                },
                {
                    type: 'div',
                    props: {
                        class: 'q-pa-md row items-start q-gutter-md',                                            
                    },
                    children:{
                        default: (slot) => {
                            const trainingDatesValues = this.jobConfiguration.trainingDates.map((dates) => {             
                                const startDate = this.ctx.moment.moment(dates.start).format('DD.MM.YYYY');                 
                                const endDate = this.ctx.moment.moment(dates.end).format('DD.MM.YYYY');                 
                                return  {
                                    type: 'NgText',
                                    props: {
                                        text: `${startDate} - ${endDate}`,
                                    },                                                                    
                                }    
                            });

                            const measureDatesValues = this.jobConfiguration.measureDates.map((dates) => {
                                const startDate = this.ctx.moment.moment(dates.start).format('DD.MM.YYYY');                 
                                const endDate = this.ctx.moment.moment(dates.end).format('DD.MM.YYYY');                                  
                                return  {
                                    type: 'NgText',
                                    props: {
                                        text: `${startDate} - ${endDate}`,
                                    },                                                                    
                                }    
                            });
                            
                            const vNodes = [
                                ...this.getCard('timeResolution', this.jobConfiguration.timeResolution),                                                                                                        
                                ...this.getCard('trainMethod', this.jobConfiguration.trainMethod),                                                                                                 
                                ...this.getCard('trainingPeriod', this.jobConfiguration.periodsDayTrain),                                                                                                 
                                ...this.getCard('retrainingPeriod', this.jobConfiguration.periodsDayRetrain),                                                                                                 
                                ...this.getCard('trainingDates', trainingDatesValues),                                                                                                 
                            ];

                            if (measureDatesValues.length > 0) {
                                vNodes.splice(5, 0, ...this.getCard('measureDates', measureDatesValues));
                            }

                            return vNodes;
                        }                        
                    }
                },
                { 
                    type: 'NgFormInput',
                    props: {
                        label: 'dependentVariable',
                        type: 'select',
                        modelValue:  this.selectedDependentVarUuid,
                        'onUpdate:modelValue': (value) => {
                            this.selectedDependentVarUuid = value;
                            this.filteredResults = this.filterResultsByDependentVariable(this.job.results, this.selectedDependentVarUuid);
                            this.getActiveExplNodesByDependentNode();
                        },
                        options: this.dependentOptions,
                    },
                },                
                {
                    type: 'NgTable',
                    props: {
                        title: 'explanatoryDatapoints',
                        rows: this.activeExplanatoryVariables,
                        columns: this.explanatoryDataColumns,
                        flat: true,
                    },             
                },
                {
                    type: 'NgDialog',
                    props: {
                        title: 'createTestEnvironment',
                        modelValue: this.dialog.testEnvironment,
                        noOk: true,
                        noCancel: true,
                        draggable: true,
                        maximizing: true,
                        'onUpdate:modelValue': (value) => {
                            this.dialog.testEnvironment = value;
                        },         
                    },
                    children: {
                        default: () => {
                            return [
                                {
                                    type: 'NgForm',
                                    props: {
                                        submitLabel: 'add',
                                        onSubmit: () => this.createTestEnvironment(),
                                    },
                                    children: {
                                        default: () => {
                                            return [
                                                {
                                                    type: 'NgFormInput',
                                                    props: {                                                        
                                                        type: 'text',
                                                        label: 'name',
                                                        col:12,
                                                        modelValue: this.testEnvironment.name,
                                                        rules: [(val) => val !== undefined && val !== '' && val !== null || this.$t('ruleNotEmpty')],
                                                        'onUpdate:modelValue': (value) => {
                                                            this.testEnvironment.name = value;
                                                        },                                                              
                                                    }
                                                },
                                                {
                                                    type: 'NgFormInput',
                                                    props: {
                                                        type: 'text',
                                                        label: 'description',
                                                        col:12,
                                                        modelValue: this.testEnvironment.description,
                                                        'onUpdate:modelValue': (value) => {
                                                            this.testEnvironment.description = value;
                                                        },                                                          
                                                    }
                                                },
                                            ]
                                        }
                                    }
                                },                                
                                
                            ];
                        }
                    }
                },                
                {
                    type: 'NgDialog',
                    props: {
                        title: 'visualize',
                        width: '1200px',
                        medium: false,
                        modelValue: this.dialog.visualizeResult,
                        noOk: true,
                        noCancel: true,
                        draggable: true,
                        maximizing: true,
                        'onUpdate:modelValue': (value) => {
                            this.dialog.visualizeResult = value;
                        },         
                    },
                    children: {
                        default: () => {
                            return [
                                {
                                    type: 'NgText',
                                    props: {
                                        text: 'applyResultTitle',
                                        look: 'h6',
                                    }
                                },
                                {
                                    type: 'NgGrid',
                                    props: {
                                        gutterX: 'sm'
                                    },
                                    children: {
                                        default:  () => {
                                            return [
                                                {
                                                    type: 'NgGridItem',
                                                    props: {
                                                        col: 6,
                                                    },
                                                    children: {
                                                        default: () => {
                                                            return [
                                                                {
                                                                    type: 'NgForm',
                                                                    props: {
                                                                        submitLabel: 'applyConfig',
                                                                        onSubmit: () => this.applyConfig(),
                                                                    },
                                                                    children: {
                                                                        default: () => {
                                                                            return [
                                                                                {
                                                                                    type: 'NgFormInput',
                                                                                    props: {
                                                                                        'data-cy': 'pageSelect',
                                                                                        label: 'testEnvironments',
                                                                                        col: 12,
                                                                                        type: 'select',
                                                                                        modelValue: this.pageUuid,
                                                                                        'onUpdate:modelValue': (value) => {
                                                                                            this.pageUuid = value;
                                                                                        },
                                                                                        rules: [(val) => val !== '' || this.$t('ruleNotEmpty')],
                                                                                        options: this.pageOptions,
                                                                                    },
                                                                                },
                                                                            ]
                                                                        }
                                                                    }
                                                                },
                                                            ]
                                                        }
                                                    }
                                                },
                                                {
                                                    type: 'NgGridItem',
                                                    props: {
                                                        col: 3,
                                                    },
                                                    children: {
                                                        default: () => {
                                                            return [
                                                                {
                                                                    type: 'NgActionButton',
                                                                    props: {
                                                                        label:'createTestEnvironment',
                                                                        'onClick': () => { 
                                                                            this.dialog.testEnvironment = true;
                                                                        },
                                                                    }
                                                                }
                                                            ]
                                                        }
                                                    }
                                                },
                                                {
                                                    type: 'NgGridItem',
                                                    props: {
                                                        col: 3,
                                                    },
                                                    children: {
                                                        default: () => {
                                                            return []
                                                        }
                                                    }
                                                }
                                            ]
                                        }
                                    }
                                },                                          
                                {
                                    type: 'NgBIChart',
                                    props: {
                                        chartDataPayload: this.chartDataPayload,
                                        disableToolbars: ['settings']
                                    }
                                },
                                {
                                    type: 'NgLoading',
                                    props: {
                                        modelValue: this.loading,
                                    }
                                },                                                                                                                             
                            ]
                        }
                    }
                },
            ];


            if (this.job.results.length > 0 && this.job.status !== 'computing') {
                const NgTableResult = {
                    type: 'NgTable',
                    props: {
                        'data-cy': 'resultTable',
                        title: 'results',
                        rows: this.filteredResults,
                        clickable: false,
                        columns: this.resultColumns,
                        flat: true,
                        pagination: {
                            sortBy: 'desc',
                            descending: false,
                            page: 1,
                            rowsPerPage: 10,
                        },
                    },
                    children: {                     
                        'body-cell-explanatoryVariables': (props) => {
                            return [
                                {
                                    type: 'NgTableTd',
                                    props: {},
                                    children: {
                                        default: () => {
                                            return this.translateExplanatoryVariables(props.row.config.explanatoryVariables)
                                            .map((explVar) => {
                                                return {
                                                    type: 'NgListItem',
                                                    props:{
                                                        style: 'min-height: 20px',
                                                    },
                                                    children: {
                                                        default: () => {
                                                            return [
                                                                explVar,
                                                            ];
                                                        },
                                                    }
                                                 };
                                            });
                                        },
                                    }
                                },   
                            ];
                        },
                        'body-cell-hyperParamTuningStart': (props) => {
                            const result = props.row;
                            return [
                                {
                                    type: 'NgTableTd',
                                    props: {
                                        class: 'text-center',
                                    },
                                    children: {
                                        default: () => {
                                            let vNodes = [
                                                {
                                                    type: 'NgActionButton',
                                                    props: {
                                                        class: result.config.tuningJobUuid === '' ? '' : 'hidden', 
                                                        flat: true,
                                                        icon: 'restart_alt',
                                                        'onClick': () => { 
                                                            this.onHyperparamterTuning(this.job, result.uuid)
                                                            .catch((err) => {
                                                                this.resetTunedJobUuid();
                                                                this.ctx.log.error(err);
                                                                if (err.response.status === 502) {
                                                                    this.ctx.notify.notifyApiError(err);
                                                                } else {
                                                                    this.ctx.notify.notify('negative', err.response.statusText);
                                                                }
                                                            });
                                                        },
                                                    }
                                                },
                                                {
                                                    type: 'NgLoading',
                                                    props: {                                                        
                                                        modelValue: result.config.tuningJobUuid === '' ? false : true,
                                                        label: '',
                                                        size: 'sm',
                                                    }
                                                },                                                                                       
                                            ];
                                            if (result.config.task.method === 'thresholdsGrouping') {
                                                vNodes = ['[n/a]'];
                                            }
                                            return vNodes;
                                        },
                                    }
                                }, 
                            ];
                        },
                        'body-cell-visualize': (props) => {
                            return [
                                {
                                    type: 'NgTableTd',
                                    props: {
                                        class: 'text-center',
                                    },
                                    children: {
                                        default: () => {
                                            return [
                                                {
                                                    type: 'NgActionButton',
                                                    props: {
                                                        flat: true,
                                                        icon: 'search',
                                                        'onClick': () => this.onVisualizeResult(props),
                                                    }
                                                },                                                    
                                            ];
                                        },
                                    }
                                },   
                            ];
                        }
                    }
                }
                vNodes.push(NgTableResult);
            }


            if (this.job.results.length === 0 && this.job.status !== 'computing') {
                const NgActionButton = {
                    type: 'NgActionButton',
                    props: {
                        label: 'startMetricComputation',
                        onClick: () => {
                            this.$emit('start:computation', this.job);
                        },
                    }
                };
                vNodes.splice(3, 0, NgActionButton)
            }

            if (this.job.status === 'computing') {
                const NgText = {
                    type: 'NgText',
                    props: {
                        text: 'computing',
                        weight: 'bold',
                    }
                };
                vNodes.splice(3, 0, NgText)
            }            
            

            vNodes.push(
                {
                    type: 'NgLoading',
                    props: {
                        modelValue: this.loading,
                    }
                },                
            );

            return vNodes;
        },
    }
  });