window.ngui(() => {
    return {
        emits: [
            'update:trainMethod',
            'update:nodeCollection',
            'update:trainDates',
            'update:trainTime',
            'update:interval',
        ],
        props: [
            'nodeCollection', // Node|ChildNode[]
            'trainDates',
            'trainMethod',
            'trainMethodOptions',
            'trainTime',
            'interval',
            'intervalOptions'
        ],
        data() {
            return {
                localTrainMethod: 'movingWindows',
                localTrainTime: this.trainTime || '00:00',
                localInterval: this.interval,
                nodeOptions: [],
                selectNode: '',
                selectedNodeIndex: 0,
                dateSelectorNodes: [],
                dateSelectorConfig: [
                    {
                        selectionMethod: 'movingWindows',
                        items: [
                            {
                                id: 'trainingDates',
                                opacity: 0.3,
                                color: '#abdbe3',
                                dates: [],
                                maxOneElement: true,
                            },
                        ],
                    },
                    {
                        selectionMethod: 'isolatedWindows',
                        items: [
                            {
                                id: 'trainingDates',
                                opacity: 0.3,
                                color: '#abdbe3',
                                dates: [],
                                maxOneElement: false,
                            },
                        ],
                    },
                ],                  
            }
        },
        methods: { 
            createBIChartNodes() {
                this.dateSelectorNodes = [];
                const endpoints = this.nodeCollection.map((node) => this.ctx.processManager.fetchEndpoint(node.pmModel));
                Promise.all(endpoints)
                .then((endpoints) => {
                    endpoints.forEach((endpoint, index) => {
                        const node = this.nodeCollection[index];
                        this.dateSelectorNodes.push({
                            node: { 
                                url: endpoint, 
                                pathes: [node.pmModel.path],
                                processManagerUuid: node.pmModel.processManagerUuid
                            },
                            interpolation: node.interpolation,
                            extInfos: node.extInfos,
                            chartOptions: node.chartOptions,
                        });
                    })
                })
                .catch((err) => {
                    this.ctx.notify.notifyApiError(err);
                });             
            },                        
        },
        beforeMount() {
            this.nodeOptions = this.nodeCollection.map((node) => ({ label: `${node.extInfos.name} ${node.pmModel.path}`, value: `${node.extInfos.name} ${node.pmModel.path}` }));
            this.selectNode = this.nodeOptions[0].value; // string
            this.createBIChartNodes();
            this.localTrainMethod = this.trainMethod;
            this.dateSelectorConfig[0].items[0].dates = this.trainDates;
            this.dateSelectorConfig[1].items[0].dates = this.trainDates;
        },
        watch: {
            dateSelectorConfig: {
                deep: true,
                handler(newValue){
                    const option = newValue.filter((opt) => opt.selectionMethod === this.localTrainMethod)[0];
                    option.items.forEach((item) => {
                      if (item.id === 'trainingDates') {
                        this.$emit('update:trainDates', item.dates);
                      }
                    });
                }
            }            
        },        
        nodes() {
            const vNodes = [];
            if (this.dateSelectorNodes.length > 0) {
                vNodes.push(
                    {                
                        type: 'NgBIDateSelector',
                        props: {
                            dateSelectorOptions: this.dateSelectorConfig,
                            selectionMethod: this.localTrainMethod,
                            nodes: this.dateSelectorNodes,
                            enableTimeInput: false,
                            time: '00:00:00',
                            interval: this.localInterval,                        
                            'onUpdate:interval': (value) => {
                                this.localInterval = value;
                                this.$emit('update:interval', value);
                            },
                            intervalOptions: this.intervalOptions,                        
                            'onUpdate:chartNodes': (dateSelectorNodes) => {
                                this.dateSelectorNodes = dateSelectorNodes;
                                dateSelectorNodes.forEach((node, index) => {
                                    this.nodeCollection[index].interpolation = node.interpolation;
                                });
                                this.$emit('update:nodeCollection', this.nodeCollection);
                            },
                            'onUpdate:selectionMethod': (selectionMethod) => {
                                this.localTrainMethod = selectionMethod;
                                this.$emit('update:trainMethod', selectionMethod);
                            }                            
                        }
                    },                                                                     
                );
            }
            return vNodes;
        },
    }
  });