window.ngui(() => {
    return {
        emits: [
            'update:modelValue',
            'submit:item',
            'remove:item',
        ],
        props: [
            'modelValue',
            'maxItems',
            'trainMethod',
        ],        
        data() {
            return {
                dateFormat: 'YYYY-MM-DD',
                defaultNode: {
                    start: '',
                    end: ''               
                },
            }
        },
        methods: { 
        },
        mounted() {            
        },
        nodes() {
            return [              
                {
                    type: 'NgFormInput',
                    props: {
                        label: this.trainMethod === 'movingWindows' ? 'movingWindowsEvaluationTimeSpan' : 'isolatedWindowsTrainTimeSpan',
                        type: 'collection',
                        enableSelectAll: false,
                        modelValue: this.modelValue,
                        maxItems: this.trainMethod === 'movingWindows' ? 1 : undefined,                        
                        'onUpdate:modelValue': (value) => {
                            this.$emit('update:modelValue', value);
                        },                            
                        'onSubmit:item': (value) => {
                            this.$emit('submit:item', value);
                        }, 
                        'onRemove:item': (value) => {
                            this.$emit('remove:item', value);
                        },                                                                                                         
                        defaultValue: this.defaultNode
                    },
                    children: {
                        form: (slot) => {
                            const item = slot.items[slot.selectedIndex];
                            return [
                                {
                                type: 'NgFormInput',
                                props: {
                                    col: 12,
                                    label: 'trainingStartDate',
                                    type: 'date-time',
                                    modelValue: item.start,
                                    mask: 'YYYY-MM-DD',
                                    format: 'YYYY-MM-DDTHH:mm:ssZ',
                                    time: false,
                                    'onUpdate:modelValue': (value) => {
                                        item.start = value;
                                    },                                      
                                    rules: [
                                        (val) => val !== '' || this.$t('ruleNotEmpty'),
                                        (val) => item.end === '' ||val < item.end || this.$t('ruleStartDateSmallThanEndDate'),
                                        (val) => this.ctx.moment.moment(val, this.dateFormat, true).isValid() || this.$t('ruleInvalidDateFormat', {'format': this.dateFormat})                                    
                                    ]
                                },
                            },   
                            {
                                type: 'NgFormInput',
                                props: {
                                    col: 12,
                                    label: 'trainingEndDate',
                                    type: 'date-time',
                                    mask: 'YYYY-MM-DD',
                                    format: 'YYYY-MM-DDTHH:mm:ssZ',
                                    time: false,                                    
                                    modelValue: item.end,
                                    'onUpdate:modelValue': (value) => {
                                        item.end = value;
                                    },
                                    rules: [
                                        (val) => {
                                            if (this.trainMethod === 'isolatedWindows') {
                                                if (val === '')  {
                                                    return this.$t('ruleNotEmpty');
                                                }
                                                if (this.ctx.moment.moment(val, this.dateFormat, true).isValid() === false) {
                                                    return this.$t('ruleInvalidDateFormat', {'format': this.dateFormat});   
                                                }
                                            }
                                            return true;
                                        },
                                    ]
                                },
                            },]                                                    
                        },
                        item: (slot) => {
                            const trainingDate = slot.item;
                            const start = this.ctx.moment.getDate(trainingDate.start);
                            const end = (trainingDate.end !== '') ? this.ctx.moment.getDate(trainingDate.end).format('DD.MM.YYYY') : this.$t('today');
                            return [
                                { 
                                    type: 'NgText',
                                    props: {
                                        text: 'Start: '  + start.format('DD.MM.YYYY') + ' – End: ' + end
                                    },                                
                                }, 
                            ]
                        },                                                                                                                         
                    },
                },
            ]
        },
    }
  });