import sys
import os
current = os.path.dirname(os.path.realpath(__file__))
first_parent = os.path.dirname(current)
sys.path.append(first_parent)
# intelli_service lies behind two parents
second_parent = os.path.dirname(first_parent) 
sys.path.append(second_parent)

from lib.Thresholds import Thresholds, METHODS_CLASS_DICT
from intelli_utilities import LOCAL_TIME_ZONE
from datetime import datetime

from metrics.metric_utils import fetch_job, get_colname_list, get_node_unique_identifier
import json
import jsonpickle
import pandas as pd

def main(args):               
    input_uuid = args['inputDataUuid']
    token = args['token']
    process_manager = args['processManager']
    custom_payload = args['customPayload']

    job_uuid = custom_payload['jobUuid']
    dependent_node_name = custom_payload['dependentNodeName']
    result_uuid = custom_payload['resultUuid']
    
    response = {}        
    with open('spool/{}/input.json'.format(input_uuid), 'r') as file:
        input_data = json.load(file)

    dataFrame = pd.DataFrame(input_data['dataFrame'])
    dataFrame = dataFrame.set_index('dateTime', drop=False)
    dataFrame.index = pd.to_datetime(dataFrame.index)                

    dataframes = []
    for date in custom_payload['trainingDates']:
        dt_start = datetime.fromisoformat(date['start']).astimezone(LOCAL_TIME_ZONE)
        dt_end = datetime.fromisoformat(date['end']).astimezone(LOCAL_TIME_ZONE)
        dataframes.append(dataFrame[dt_start:dt_end])
    train_data_df = pd.concat(dataframes)

    test_data_df = pd.DataFrame()
    if len(custom_payload['measureDates']) > 0:
        dataframes = []
        for date in custom_payload['measureDates']:
            dt_start = datetime.fromisoformat(date['start']).astimezone(LOCAL_TIME_ZONE)
            dt_end = datetime.fromisoformat(date['end']).astimezone(LOCAL_TIME_ZONE)
            dataframes.append(dataFrame[dt_start:dt_end])
        test_data_df = pd.concat(dataframes)
    
    job = fetch_job(job_uuid, token, process_manager)
    job_config = job['config']
        
    # Pick selected result
    results = list(filter(lambda r: r['uuid'] == result_uuid, job['results']))
    if len(results) == 0:
        raise ValueError('Result uuid: {} not found in job uuid: {}'.format(result_uuid, job_uuid))
    result = results[0]
    result_config = result['config']
    
    thresholds = Thresholds()
    
    model = None    
    if result_config['task']['method'] != 'thresholdsGrouping':
        grid_params = None
        if 'gridSearch' in result_config:
            if result_config['gridSearch']['tunedMetric'] < result_config['metric']:
                if result_config['gridSearch']['bestParams'] != '':
                    grid_params = result_config['gridSearch']['bestParams']
                else:
                    grid_params = jsonpickle.decode(result_config['gridSearch']['bestParamsEncoded'])
        
        model = thresholds.get_model(result_config['task']['model'])()
        if grid_params:    
            model = thresholds.get_model(result_config['task']['model'])(**grid_params)
        
        
    explanatory_var_name_list = get_colname_list(result_config['explanatoryVariables'])
    dependent_df_colname = get_node_unique_identifier(result_config['dependentVar']['pmModel'])
    threshold_instance = METHODS_CLASS_DICT[result_config['task']['method']](dependent_variables=[dependent_df_colname], explanatory_vars=explanatory_var_name_list, use_stats=result_config['stats'], model=model)

    if job_config['trainMethod'] == 'isolatedWindows':        
        # Train model
        thresholds_df = thresholds.train_static_data(threshold_instance, train_data_df, test_data_df)

    if job_config['trainMethod'] == 'movingWindows':
        thresholds_df, _, _ = thresholds.train_rolling_data(threshold_instance, train_data_df, job_config['periodsDay']['train'], job_config['periodsDay']['retrain'], True)

    thresholds_df = thresholds.bake_thresholds(thresholds_df)
    thresholds_df = thresholds_df[(thresholds_df['lower'].isnull()) == False & (thresholds_df['upper'].isnull() == False)]    

    response = {
        'line': [
            { 
                'id': 'expectedValue',
                'name': 'expectedValue',
                'x': thresholds_df['dateTime'].values.tolist(),
                'y': thresholds_df['Y_pred'].values.tolist(),
                'color': '#47b547',
                'yaxis': dependent_node_name
            }
        ],
        'rangeArea': [], 
        'annotations': {},
        'thresholdsDataFrame' : thresholds_df.to_dict('records')
    }

    
    return response    
    
if __name__ == "__main__":
    response = main(runpy_args)