// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()

window.ngui(() => {
  return {
    props: ['instanceUuid', 'serviceUuid'],
    emits: ['submit', 'cancel'],
    beforeCreate() {
      components.setParent(this)
    },

    created() {
    },
    
    // Listen on mounted event...
    mounted() {
      // logic
      if (this.instanceUuid) {
        this.ctx.processManager.fetchInstanceSettings(this.processManagerUuid, this.instanceUuid)
          .then((instance) => {
            this.form = instance
            this.hasDMS = !!this.form.processes.find((proc) => proc.name === 'DMS')
          })
      }
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        withDMS: !!this.$attrs.withDMS,
        isNew: !this.instanceUuid,
        processManagerUuid: this.serviceUuid || this.$route.query.service,
        hasDMS: false,
        form: {
          uuid: '',
          name: '',
          description: '',
          enabled: true,
          processes: [],
        },
      }
    },

    // Define methods for the ui...
    methods: {
      submit() {
        if (this.withDMS) {
          this.form.enabled = true
        }
        this.ctx.processManager.putInstanceSettings(this.processManagerUuid, this.form.uuid, this.form)
          .then((instanceMessage) => {
            this.form.uuid = instanceMessage.instanceUuid
            if (this.withDMS) {
              this.ctx.processManager.fetchTemplates(this.processManagerUuid, this.form.uuid)
                .then((templates) => {
                  const dms = templates.find((template) => template.name === 'DMS')
                  this.form.processes.push(dms)
                  this.ctx.processManager.putInstanceSettings(this.processManagerUuid, this.form.uuid, this.form)
                    .then(() => {
                      this.ctx.processManager.putInstanceAction(this.processManagerUuid, this.form.uuid, 'Start')
                      this.$emit('submit', instanceMessage, this.form, this.isNew)
                    })
                })
            } else {
              this.$emit('submit', instanceMessage, this.form, this.isNew)
            }
          })    
      },
      cancel() {
        this.$emit('cancel')
      },
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        components.form().append(() => [
          components.caption('general'),
          components.input('ID', 'text', this.form.uuid, (val) => this.form.uuid = val).attrs({ readonly: true }),
          components.input('name', 'text', this.form.name, (val) => this.form.name = val).attrs({ required: true }),
          components.input('description', 'text', this.form.description, (val) => this.form.description = val),
          components.input('autoStart', 'checkbox', this.form.enabled, (val) => this.form.enabled = val),
          components.renderIf(!this.hasDMS, components.input('addDMS', 'checkbox', this.withDMS, (val) => this.withDMS = val)),
        ]).on({
          onSubmit: () => this.submit(),
          onCancel: () => this.cancel(),
        })
      ]
    },
  }
})