// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()

window.ngui(() => {
  return {
    props: ['instanceUuid', 'processUuid'],
    emits: ['submit', 'cancel'],
    beforeCreate() {
      components.setParent(this)
    },

    created() {
      this.ctx.injecter.set('serviceUuids', [this.processManagerUuid])
    },
    
    // Listen on mounted event...
    mounted() {
      // logic      
      if (this.processUuid) {
        this.ctx.processManager.fetchProcessSettings(this.processManagerUuid, this.instanceUuid, this.processUuid)
          .then((process) => {
            this.form = process
          })
      }
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        processManagerUuid: this.$route.query.service,
        form: {
          uuid: '',
          name: '',
          description: '',
          enabled: false,
        },
      }
    },

    // Define methods for the ui...
    methods: {
      submit() {
        this.ctx.processManager.putProcessSettings(this.processManagerUuid, this.instanceUuid, this.form.uuid, this.form)
          .then((process) => {
            this.$emit('submit', process)
          })    
      },
      cancel() {
        this.$emit('cancel')
      }
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        components.form().append(() => [
          components.caption('general'),
          components.input('ID', 'text', this.form.uuid, (val) => this.form.uuid = val).attrs({ readonly: true }),
          components.input('name', 'text', this.form.name, (val) => this.form.name = val).attrs({ readonly: true }),
          components.input('description', 'text', this.form.description, (val) => this.form.description = val),
          components.input('autoStart', 'checkbox', this.form.enabled, (val) => this.form.enabled = val),
        ]).on({
          onSubmit: () => this.submit(),
          onCancel: () => this.cancel(),
        })
      ]
    },
  }
})