// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()

window.ngui(() => {
  return {
    props: [],
    emits: ['click:endpoint'],
    beforeCreate() {
      components.setParent(this)
    },

    // Listen on mounted event...
    mounted() {
      // logic     
      this.ctx.workspace.toService([this.processManagerUuid]).fetchWorkspace(this.$attrs.workspaceUuid)
        .then((workspace) => {
          this.workspace = workspace
          this.fetchInstances()
        })
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        name: this.$attrs.name || 'DMS',
        processManagerUuid: this.$attrs.serviceUuid || this.$route.query.service,
        workspace: {
          name: '',
          description: '',
          uuid: '',
          instances: [],
          promosnts: [],
        },
        instances: [],
        loaded: false,
      }
    },

    // Define methods for the ui...
    methods: {
      onSelected(inst, settings) {
        // load endpoint
        // emit endpoint
        this.ctx.processManager.toService([this.processManagerUuid]).fetchServerByName(
          inst.processManagerUuid,
          inst.instanceUuid, 
          this.name,
          false,
          false,
          true,
          this.name ? undefined : 'DMS',
        )
          .then((server) => {
            const endpoint = this.ctx.processManager.getEndpoint(server)
            this.$emit('click:endpoint', { 
              endpoint, 
              name: settings.name, 
              instanceUuid: inst.instanceUuid, 
              processManagerUuid: inst.processManagerUuid, 
              processUuid: server.processUuid,
              nt: false 
            })
          })
        .catch((err) => {
            components.notify().negative(err)
        })
      },

      onSelectedNT(inst) {
        // load endpoint
        // emit endpoint
        this.$emit('click:endpoint', { endpoint: inst.url, name: inst.name, instanceUuid: '', nt: true  })
      },

      getStatusText(instance) {
        const proc = instance.processes.find((p) => p.name === this.name)
        if (proc) {
          return proc.statusText
        }

        return instance.statusText
      },

      fetchInstances() {
        this.loaded = false
        const promises = []
        this.workspace.instances.forEach((inst) => {
          promises.push(this.ctx.processManager.toService([this.processManagerUuid]).fetchInstanceStatus(inst.processManagerUuid, inst.instanceUuid))
        })

        Promise.all(promises)
          .then((instances) => {
            this.loaded = true
            this.instances = instances.map((instances) => instances)
          })
      }
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        components.card(this.name).append(() => [
            components.renderIf(this.loaded, components.list('ProMoS NG').append(() => [
              ...this.workspace.instances.map((instance, idx) => components.item(this.instances[idx].name, this.getStatusText(this.instances[idx])).attrs({ clickable: true }).on({
                onClick: () => this.onSelected(instance, this.instances[idx]),
                disable: () => this.onSelected(instance, this.instances[idx]),
              }))
            ])),
            components.spacing(['mt-md']).append(() => [
                components.renderIf(this.workspace.promosnts && this.workspace.promosnts.length, components.list('ProMoS NT').append(() => [
                ...this.workspace.promosnts.filter((instance) => instance.type === undefined || instance.type === this.$attrs.name).map((instance) => components.item(instance.name, instance.url).attrs({ clickable: true }).on({
                  onClick: () => this.onSelectedNT(instance),
                }))
              ]))
            ])
        ])
      ]
    },
  }
})