export default function useScripts() {
  
	function calculateOffsetFromRatio(ratio) {
		return ratio / 100;
	}
	
	function interpolateColors(color1, color2, factor) {
		const hexToRgb = hex => {
			const bigint = parseInt(hex.slice(1), 16);
			return [
			  (bigint >> 16) & 255,
			  (bigint >> 8) & 255,
			  bigint & 255
			];
		};

		const rgbToHex = rgb => {
			return '#' + rgb.map(x => x.toString(16).padStart(2, '0')).join('');
		};

		const rgb1 = hexToRgb(color1);
		const rgb2 = hexToRgb(color2);

		const r = Math.round(rgb1[0] + factor * (rgb2[0] - rgb1[0]));
		const g = Math.round(rgb1[1] + factor * (rgb2[1] - rgb1[1]));
		const b = Math.round(rgb1[2] + factor * (rgb2[2] - rgb1[2]));

		return rgbToHex([r, g, b]);
	}

	function interpolateTemperatureColor(temp, colors, minTemp, maxTemp) {
		let factor = 0;
		
		if (colors.length < 1) {
			return '#000000';
		}

		if (minTemp === maxTemp) {
			return colors[0];
		} else {
			factor = Math.min(Math.max((temp - minTemp) / (maxTemp - minTemp), 0), 1);
		}

		let color1 = colors[0];
		let color2 = colors.length < 2 ? colors[0] : colors[1];

		for (let i = 0; i < colors.length - 1; i++) {
			const stop1 = colors[i];
			const stop2 = colors[i + 1];

			if (temp >= stop1.offset * (maxTemp - minTemp) && temp <= stop2.offset * (maxTemp - minTemp)) {
			  color1 = stop1;
			  color2 = stop2;
			  break;
			}
		}

		return interpolateColors(color1, color2, factor);
	}

	return {
		interpolateTemperatureColor,
		calculateOffsetFromRatio,
	}
}