// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()
import useProcessManager from '/service/rest/v1/files/data/public/js/useProcessManager.js'
const processManager = useProcessManager()
const StatusTable = window.nguiAsyncImport('public/ui/main/process-manager/instances/processes/status/index.js')
const GenStxGenerateDialog = window.nguiAsyncImport('public/forms/gen-stx.js')

window.ngui(() => {
  return {
    mixins: [
      processManager.mixin(),
    ],
    // Listen on mounted event...
    beforeCreate() {
      components.setParent(this)
      processManager.setParent(this)
    },

    watch: {
      // whenever question changes, this function will run
      node(newNode) {
        this.setQueryByNode(newNode)
      },
      
      endpoint(val) {
        if (!val) {
          return
        }
        this.showTreeMenu = false
		
        clearTimeout(this.showTreeMenuTimeout)
        // let a bit time before loading tree menu to calm down the communication
        this.showTreeMenuTimeout = setTimeout(() => {
          this.showTreeMenu = true
        }, 250)
      },
    },

    unmounted() {
      clearInterval(this.processStatusIntervalId)
    },

    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        processStatusIntervalId: 0,
        processName: this.$attrs.processName || 'DMS',
        loading: false,
        version: '0.0.1',
        showTreeMenu: false,
        showTreeMenuTimeout: null,
        processManagerUuid: this.$route.query.service,
        serviceUuid: this.$route.query.service,
        serviceUuids: [],
        workspaceUuid: this.$route.query.workspace,
        instanceUuid: this.$route.query.instance,
        processUuid: this.$attrs.processUuid || this.$route.query.process,
        genSTXUuid: '',
        genSTXInstanceUuid: '',
        endpoint: '',
        name: '',
        isNt: false,
        isNG: true,
        select: false,
        selected: [],
        dialog: {
          open: false,
          genStxStatus: false,
          genStxGenerate: false,
        },
        node: {
          path: this.$route.query.path || '',
        },
        settings: {
          digits: 3,
          panes: {
            name: 'panes',
            horizontal: false,
            panes: [
              {
                slot: 'left',
                size: 20,
              },
              {
                slot: 'right',
              },
            ],
          }
        },
        workspace:  {
          name: '',
          instances: []
        },
      }
    },

    // Define methods for the ui...
    methods: {
      pingStatus() {
        clearInterval(this.processStatusIntervalId)
        if (this.processUuid) {
          this.ctx.processManager.fetchProcessStatus(this.processManagerUuid, this.instanceUuid, this.processUuid)
            .then((status) => {
              this.processStatus = status
            })
            .finally(() => {
              this.processStatusIntervalId = setTimeout(() => {
                this.pingStatus()
              }, 2000)         
            })

        }

      },
      setQueryByNode(node) {
        this.ctx.routing.replace({
          query: {
            ...this.ctx.routing.route.query,
            path: node.path
          }
        })
      },
      getFileMenu() {
        return [
          {
            id: 'open',
            label: 'open',
            disable: !!components.router().query.instance,
            click: () => this.dialog.open = true,
            permissions: this.ctx.globals.getPermissions(['dmsRead']),
          },
          {
            id: 'save',
            label: 'save',
            separator: true,
            click: () => this.dialog.save = true,
            permissions: this.ctx.globals.getPermissions(['dmsCreate']),
          },
          {
            id: 'import',
            label: 'import',
            disable: !this.isNG,
            click: () => this.dialog.import = true,
            permissions: this.ctx.globals.getPermissions(['dmsCreate']),
          },
          {
            id: 'export',
            label: 'export',
            separator: true,
            disable: !this.isNG,
            click: () => this.dialog.export = true,
            permissions: this.ctx.globals.getPermissions(['dmsRead']),
          },
          {
            id: 'search',
            label: 'search',
            separator: true,
            click: () => this.dialog.search = true,
            permissions: this.ctx.globals.getPermissions(['dmsRead']),
          },
          {
            id: 'settings',
            label: 'settings',
            click: () => this.dialog.settings = true,
            separator: true,
            permissions: this.ctx.globals.getPermissions(['dmsCreate']),
          },
          {
            id: 'endpoint',
            label: this.endpoint,
            icon: 'content_copy',
            click: () => this.ctx.clipboard(this.endpoint, this.$t),
          },
        ]
      },

      getEditMenu() {
        return [
          {
            id: 'add',
            label: 'add',
            click: () => this.dialog.add = true,
            permissions: this.ctx.globals.getPermissions(['dmsCreate']),
          },
          {
            id: 'rename',
            label: 'rename',
            click: () => this.dialog.rename = true,
            permissions: this.ctx.globals.getPermissions(['dmsUpdate']),
          },
          {
            id: 'copy',
            label: 'copy',
            click: () => this.dialog.copy = true,
            permissions: this.ctx.globals.getPermissions(['dmsCreate']),
          },
          {
            id: 'type',
            label: 'type',
            separator: true,
            click: () => this.dialog.changeType = true,
            permissions: this.ctx.globals.getPermissions(['dmsUpdate']),
          },
          {
            id: 'select',
            label: this.dialog.select ? 'selectOff' : 'select',
            click: () => {
              this.dialog.select = !this.dialog.select
              this.selected = []
            },
            permissions: this.ctx.globals.getPermissions(['dmsRead']),
          },
          {
            id: 'deleteSelected',
            label: 'deleteSelected',
            disable: this.selected.length <= 0,
            separator: true,
            click: () => this.dialog.deleteSelected = true,
            permissions: this.ctx.globals.getPermissions(['dmsDelete']),
          },
          {
            id: 'copyPath',
            label: 'copyPath',
            click: () => {
              this.ctx.clipboard(this.node.path)
            },
            permissions: this.ctx.globals.getPermissions(['dmsRead']),
          },
        ]
      },

      getViewMenu() {
        return [
          {
            id: 'process',
            label: 'process',
            icon: 'widgets',
            click: () => {
              components.router().push('/process-manager/instances/processes/process', { 
                service: this.processManagerUuid,
                instance: this.instanceUuid,
                process: this.processUuid,
                name: this.processName,
              })
            },
            permissions: this.ctx.globals.getPermissions(['processUpdate']),
          },
          {
            id: 'logs',
            label: 'logs',
            icon: 'description',
            disable: !this.processUuid,
            click: () => {
              components.router().push('/process-manager/instances/processes/logs/index', { 
                service: this.serviceUuid,
                instance: this.instanceUuid,
                process: this.processUuid,
              })
            },
            permissions: this.ctx.globals.getPermissions(['logRead']),
          },

          {
            id: 'status',
            label: 'status',
            icon: 'checklist',
            disable: !this.processUuid,
            click: () => {
              components.router().push('/process-manager/instances/processes/status/index', { 
                service: this.serviceUuid,
                instance: this.instanceUuid,
                process: this.processUuid,
              })
            },
            permissions: this.ctx.globals.getPermissions(['statusRead']),
          },

          {
            id: 'module',
            label: 'module',
            icon: 'view_module',
            disable: true,
            click: () => {
              components.router().push(`/modules/${this.processSettings.name}/index`, { 
                service: this.serviceUuid,
                instance: this.instanceUuid,
                process: this.processUuid,
              })
            },
            permissions: this.ctx.globals.getPermissions(['processesRead']),
          },
        ]
      },

      getDisplayMenu() {
        return [
          {
              id: 'datapoint',
              label: 'datapoint',
              separator: true,
              click: () => this.dialog.datapoint = true,
              permissions: this.ctx.globals.getPermissions(['dmsRead']),
          },
          {
            id: 'newTab',
            label: 'newTab',
            click: () => {
              this.ctx.routing.openURL(`/service/app/#${this.ctx.routing.route.fullPath}`)
            },
          },
        ]
      },

      getGenSTXMenu() {
        return [
          {
            id: 'genSTXGenerate',
            label: 'generate',
            icon: 'code',
            disable: !this.genSTXUuid,
            click: () => {
              this.dialog.genStxGenerate = true
            },
            permissions: this.ctx.globals.getPermissions(['processesUpdate']),
          },
          {
            id: 'genSTXStatus',
            label: 'status',
            icon: 'checklist',
            disable: !this.genSTXUuid,
            click: () => {
              this.dialog.genStxStatus = true
            },
            permissions: this.ctx.globals.getPermissions(['statusRead']),
          },
          {
            id: 'genSTXLogs',
            label: 'logs',
            icon: 'description',
            disable: !this.genSTXUuid,
            click: () => {
              window.processManager.toLogs(
                this.serviceUuid,
                this.genSTXInstanceUuid,
                this.genSTXUuid)
            },
            permissions: this.ctx.globals.getPermissions(['changelogRead']),
          },          
          {
            id: 'genSTXConfiguration',
            label: 'configuration',
            icon: 'settings',
            disable: !this.genSTXUuid,
            click: () => {
              window.processManager.toConfiguration(
                this.serviceUuid, 
                this.genSTXInstanceUuid, 
                this.genSTXUuid,
                'GenSTX')
            },
            permissions: this.ctx.globals.getPermissions(['processesUpdate']),
          },          
        ]
      },
      
      setLastInstance(processManagerUuid, instanceUuid, processUuid) {
        this.ctx.rest.lsSetJSON(`${this.processName.toLowerCase()}:${this.workspaceUuid}:lastInstance`, { processManagerUuid, instanceUuid, processUuid })
      },

      getLastInstance() {
        return this.ctx.rest.lsGetJSON(`${this.processName.toLowerCase()}:${this.workspaceUuid}:lastInstance`)
      },

      saveSettings(disableNotify) {
        clearTimeout(this.storeTimeoutId)
        const copyEndpoint = this.endpoint
        if (!disableNotify) {
          this.endpoint = ''
        }
        this.storeTimeoutId = setTimeout(() => {
          this.ctx.rest.lsSetJSON(`${this.processName.toLowerCase()}:settings`, this.settings)
          if (!disableNotify) {
            components.notify().positive(this.ctx.i18n.i18n.t('saved'))  
            this.endpoint = copyEndpoint
          }
        }, 500)
        this.dialog.settings = false
      },

      deleteSelected() {
        const nodes = this.selected.map((node) => ({ path: node.path, recursive: true }))
        this.ctx.dms.post({url: this.endpoint, payload: {
          delete: nodes,
        }})
        .then((res) => {
          components.notify().positive(res)
          this.selected = []
        })
        .catch((err) => {
          components.notify().negative(err)
        })
      },

      loadPromos(obj = { endpoint: '', processManagerUuid: '', instanceUuid: '', processUuid: '', nt: false }) {
        // Let a bit time to rerender
        clearInterval(this.processStatusIntervalId)
        
        this.node.path = ''
        this.loading = true
        this.endpoint = ''
        this.isNt = obj.nt
        
        if (obj.nt) {       
          setTimeout(() => {
            this.serviceUuids = []
            this.instanceStatus.name = obj.name
            this.endpoint = obj.endpoint
            this.loading = false
          }, 200)
          return
        }
        
        if (obj.processManagerUuid && obj.instanceUuid && obj.processUuid) {
          this.setLastInstance(obj.processManagerUuid, obj.instanceUuid, obj.processUuid)
        }

        setTimeout(() => {
          this.init()
        }, 200)
      },

      async init() {
        clearInterval(this.processStatusIntervalId)
        this.endpoint = ''
        this.genSTXUuid = ''
        this.genSTXInstanceUuid = ''
        this.loading = true
        let newEndpoint = ''
        const settings = this.ctx.rest.lsGetJSON(`${this.processName.toLowerCase()}:settings`) || this.settings
        if (!settings.panes) {
          settings.panes = this.settings.panes
        }

        settings.panes.horizontal = this.ctx.platform.mobile
        this.settings = settings
        
        if (this.workspaceUuid && !this.workspace.uuid) {
          try {
            this.workspace = await this.ctx.workspace.toService([this.serviceUuid]).fetchWorkspace(this.workspaceUuid)
            this.name = this.workspace.name
          } catch (err) {
            this.ctx.notify.notifyApiError(err)
          }
        }

        const instance = this.getLastInstance()

        if (instance) {
          if (this.workspace && this.workspace.instances && !!this.workspace.instances.find((item) => item.instanceUuid === instance.instanceUuid)) {
            this.processManagerUuid = instance.processManagerUuid
            this.instanceUuid = instance.instanceUuid
            this.processUuid = instance.processUuid
          }
        }

        
        if (this.workspace.instances && this.workspace.instances.length && !this.instanceUuid) {
          const [item] = this.workspace.instances
          this.processManagerUuid = item.processManagerUuid
          this.instanceUuid = item.instanceUuid
          try {
            if (this.processManagerUuid !== this.serviceUuid) {
              this.ctx.processManager.toService([this.serviceUuid])
            }
            const instance = await this.ctx.processManager.fetchInstance(this.processManagerUuid, this.instanceUuid)
            const process = instance.processes.find((proc) => proc.name === this.processName)
            if (process) {
              this.processUuid = process.uuid
            }
          } catch (e) {
            // nothing to do
          }
        }

        if (this.instanceUuid && this.processUuid) {
          try {  
            if (this.processManagerUuid !== this.serviceUuid) {
              this.ctx.processManager.toService([this.serviceUuid])
            }
            const res = await this.ctx.processManager.fetchProcess(
				this.processManagerUuid,
                this.instanceUuid,
                this.processUuid,
                this.processName,
              )
            this.instanceStatus = res.instance
            this.processStatus = res.process
            this.processManager = res.processManager
            this.processServer = res.server
            newEndpoint = this.ctx.processManager.getEndpoint(this.processServer)
          } catch (err) {
            this.ctx.notify.notifyApiError(err)
          }
        }

        this.serviceUuids = [this.serviceUuid || '', this.processManagerUuid]
        this.loading = false

        // Get GenSTX status server
        if (this.processName === 'DMS') {
          try {
            const genstxRes = await this.ctx.processManager.toService(this.serviceUuids).fetchServers(
              this.processManagerUuid, 
              'GenSTX',
              false,
              true,
              'statusApi',
            )
  
            if (genstxRes.instances && genstxRes.instances.length) {
              if (genstxRes.instances[0].processes && genstxRes.instances[0].processes.length) {
                this.genSTXInstanceUuid = genstxRes.instances[0].uuid
                this.genSTXUuid = genstxRes.instances[0].processes[0].uuid
              }
            }
          } catch (e) {
            // nothing to do
          }  
        } else {
          this.genSTXUuid = ''
        }

        this.endpoint = newEndpoint
        this.pingStatus()
      },

      getBreadcrumb() {
        if (this.workspaceUuid) {
          return processManager.getPMBreadcrumbs([
            ...processManager.getPMWorkspaceBreadcrumb(),
            {
              label: this.processName,
            },
          ])
        }

        return processManager.getPMBreadcrumbs(this.processManagerUuid, this.instanceUuid, this.processUuid, 'module')
      }
    },

    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        components.page([30, 18], () => this.init()).append(() => [
          components.renderIf(!this.$attrs.disableBread, components.breadcrumbs(this.getBreadcrumb())),
          components.loading('loading', this.loading, (val) => this.loading = val).attrs(),
          components.bar(this.instanceStatus.name || this.processName).append('end', () => [
              components.renderIf(!this.isNt, components.badge(this.processStatus.statusText).attrs({
                color: this.processStatus.status === 2 ? 'positive' : 'warning',
              }))
            ])
            .createMenu('file', this.getFileMenu())
            .createMenu('edit', this.getEditMenu())
            .createMenu('view', this.getViewMenu())
            .createMenu('display', this.getDisplayMenu())
            .createMenu('genSTX', '', this.getGenSTXMenu(), [], !this.genSTXUuid || this.processName !== 'DMS'),
          
          components.section().append(() => [
            components.renderIf(!this.endpoint && this.workspaceUuid && !this.loading, components.card().append(() => [
              components.section().append(() => [
                components.text('noProjectSelected').attrs(),
              ]),
              components.section().append(() => [
                components.button('open').on({ onClick: () => this.dialog.open = true }),
              ]),
            ])),

            
            components.renderIf(this.endpoint && this.ctx.platform.browser, components.card().attrs({ flat: false, class: 'q-pa-sm' }).append(() => [
               components.element('NgSplitter').attrs({
                 modelValue: this.settings.panes,
               }).on({
                 'onUpdate:modelValue': (val) => {
                   this.settings.panes = val
                   this.saveSettings(true)
                 },
               }).append('left', () => [
                 components.renderIf(this.showTreeMenu, components.element('NgDMSTreeMenu').attrs({
                   node: this.node,
                   url: this.endpoint,
                   serviceUuids: this.serviceUuids,
                   workspaceUuid: this.workspaceUuid,
                   disableContextMenu: false,
                   style: {
                     maxHeight: this.ctx.platform.mobile ? '25vh' : '79vh',
                     overflow: 'auto',
                   },
                   flat: false,
                 }).on({
                   'onClick:node': (val) => this.node = val,
                 })
                 ),
               ]).append('right', () => [
                 components.renderIf(this.endpoint, components.element('NgDMSTable').attrs({
                   serviceUuids: this.serviceUuids,
                   workspaceUuid: this.workspaceUuid,
                   node: this.node,
                   url: this.endpoint,
                   select: this.dialog.select,
                   selected: this.selected,
                   disableTitle: true,
                   maxHeight: '79vh',
                   flat: false,
                 }).on({
                   'onUpdate:selected': (val) => this.selected = val,
                   'onClick:node': (val) => val.path !== this.node.path ? (this.node = val) : undefined,
                   'onVersion:ng': (val) => this.isNG = val,
                 }))
               ])
            ])),
            components.renderIf(this.endpoint && !this.ctx.platform.browser, components.element('div').append(() => [
              components.element('NgDMSTable').attrs({
                 serviceUuids: this.serviceUuids,
                 workspaceUuid: this.workspaceUuid,
                 node: this.node,
                 url: this.endpoint,
                 select: this.dialog.select,
                 selected: this.selected,
                 disableTitle: true,
                 maxHeight: '80vh',
               }).on({
                 'onUpdate:selected': (val) => this.selected = val,
                 'onClick:node': (val) => this.node = val
               })
            ])),
            // Dialogs
            components.formDialog('open', this.dialog.open, (val) => this.dialog.open = val, 'public/forms/select-endpoint.js').formOn({
              'onClick:endpoint': (obj) => {
                this.dialog.open = false
                this.loadPromos(obj)
              },
              onCancel: () => this.dialog.open = false,
            }).formAttrs({
              workspaceUuid: this.workspaceUuid,
              processManagerUuid: this.serviceUuid,
              name: this.processName, 
            }),

            components.dialogConfirm('save', this.dialog.save, (val) => this.dialog.save = val).on({
              onOk: () => this.ctx.dms.save(this.endpoint),
            }),

            components.dialog('', this.dialog.import, (val) => this.dialog.import = val).append(() => [
              components.element('NgDMSNodeImport').attrs({
                 node: this.node,
                 url: this.endpoint,
                 serviceUuids: this.serviceUuids,
                 workspaceUuid: this.workspaceUuid,
               })
            ]),

            components.dialog('', this.dialog.export, (val) => this.dialog.export = val).append(() => [
              components.element('NgDMSNodeExport').attrs({
                 node: this.node,
                 url: this.endpoint,
                 serviceUuids: this.serviceUuids,
                 workspaceUuid: this.workspaceUuid,
               })
            ]),

            components.dialog('', this.dialog.search, (val) => this.dialog.search = val).append(() => [
              components.element('NgDMSSearch').attrs({
                 node: this.node,
                 url: this.endpoint,
                 serviceUuids: this.serviceUuids,
                 workspaceUuid: this.workspaceUuid,
               }).on({
                 'onClick:node': (val) => {
                   this.node = val
                   this.dialog.search = false
                 }
               })
            ]),

            components.dialog('', this.dialog.settings, (val) => this.dialog.settings = val).append(() => [
              components.form('settings').append(() => [
                components.input('endpoint', 'text', this.endpoint, (val) => this.endpoint = val).attrs({ readonly: true }),
                components.input('digits', 'number', this.settings.digits, (val) => this.settings.digits = val)
              ]).on({
                onSubmit: () => this.saveSettings(),
              })
            ]),

            // Edit
            components.dialog('', this.dialog.add, (val) => this.dialog.add = val).append(() => [
              components.element('NgDMSNodeAdd').attrs({
                 node: this.node,
                 url: this.endpoint,
                 serviceUuids: this.serviceUuids,
                 workspaceUuid: this.workspaceUuid,
               })
            ]),

            components.dialog('', this.dialog.rename, (val) => this.dialog.rename = val).append(() => [
              components.element('NgDMSNodeRename').attrs({
                 node: this.node,
                 url: this.endpoint,
                 serviceUuids: this.serviceUuids,
                 workspaceUuid: this.workspaceUuid,
               })
            ]),

            components.dialog('', this.dialog.copy, (val) => this.dialog.copy = val).append(() => [
              components.element('NgDMSNodeCopy').attrs({
                 node: this.node,
                 url: this.endpoint,
                 serviceUuids: this.serviceUuids,
                 workspaceUuid: this.workspaceUuid,
               })
            ]),

            components.dialog('', this.dialog.changeType, (val) => this.dialog.changeType = val).append(() => [
              components.element('NgDMSNodeChangeType').attrs({
                 node: this.node,
                 url: this.endpoint,
                 serviceUuids: this.serviceUuids,
                 workspaceUuid: this.workspaceUuid,
               })
            ]),

            components.dialogDelete(this.selected.map((node) => node.path).join(', '), this.dialog.deleteSelected, (val) => this.dialog.deleteSelected = val).on({
              onOk: () => this.deleteSelected(),
            }),
            
            // View
            components.dialog('', this.dialog.datapoint, (val) => this.dialog.datapoint = val).append(() => [
              components.element('NgDMSNodeExtInfos').attrs({
                 node: this.node,
                 url: this.endpoint,
                 serviceUuids: this.serviceUuids,
                 workspaceUuid: this.workspaceUuid,
               })
            ]),

            components.dialog('generate', this.dialog.genStxGenerate, (val) => this.dialog.genStxGenerate = val).append(() => [
              components.element(GenStxGenerateDialog).attrs({
                name: 'GenSTX',
                serviceUuids: this.serviceUuids,
                instance: this.instanceUuid,
                processInstance: this.genSTXInstanceUuid,
                process: this.genSTXUuid,
                dmsEndpoint: this.endpoint,
               })
            ]),

            components.dialog('status', this.dialog.genStxStatus, (val) => this.dialog.genStxStatus = val).append(() => [
              components.element(StatusTable).attrs({
                name: 'GenSTX',
                serviceUuids: this.serviceUuids,
                instance: this.genSTXInstanceUuid,
                process: this.genSTXUuid,
                statusIds: [''],
               })
            ]),
          ]),
        ]),
      ]
    },
  }
})

