import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()
// import 'https://cdn.jsdelivr.net/npm/sortablejs@1.15.0/Sortable.min.js'
// import 'https://cdn.jsdelivr.net/npm/frappe-datatable@1.17.3/dist/frappe-datatable.min.css'
// import 'https://cdn.jsdelivr.net/npm/frappe-datatable@1.17.3/dist/frappe-datatable.min.js'

await window.nguiIncludeScript('https://cdn.jsdelivr.net/npm/handsontable/dist/handsontable.full.min.js')
await window.nguiIncludeStyle('https://cdn.jsdelivr.net/npm/handsontable/dist/handsontable.full.min.css')

// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        breads: [],
        newValue: '',
        valueType: '',
        link: {
          processManagerUuid: '',
          instanceUuid: '',
          processUuid: '',
          path: '',
        },
        response: null,
        datatable: null,
        columns: [],
        data: [],
        options: {
          licenseKey: 'non-commercial-and-evaluation',
          data: [['', '', '', '']],
          columns: [
            { data: 0, type: "text" },
            { data: 1, type: "text" },
            { data: 2, type: "text" },
            { data: 3, type: "checkbox" },
          ],
          colHeaders: [
            'path',
            'value',
            'type',
            'hasChild',
          ],
          rowHeaders: true,
          height: 'auto',
          width: '100%',
          colWidths: 250,
          manualColumnResize: true,
        },
      }
    },

     // Listen on mounted event...
    mounted() {
      // logic
      components.setParent(this)
    },
    
    // Define methods for the ui...
    methods: {
      init() {
        setTimeout(() => {
          const container = document.querySelector('#datatable');
          this.datatable = new Handsontable(container, this.options);

          this.datatable.addHook('afterOnCellMouseDown', (ev) => {
            const selection = this.datatable.getSelected()[0];
            const [rowIndex, colIndex] = selection
            if (colIndex === 0) {
              const path = this.datatable.getValue()
              this.fetchData(path)
            }
          })

          this.initEvent()
        }, 500)
      },

      initEvent() {  
        this.dms.on('message', (res) => {
          if (res && res.payload && res.payload.event) {
            const { event } = res.payload
            let hasChanges = false
            event.forEach((node) => {
              this.data.forEach((row) => {
                const [path] = row
                if (node.path === path) {
                  row[1] = node.value
                  hasChanges = true
                }
              })
            })

            this.datatable.render()
          }
        })
    
        this.dms.connect();
      },

      createBreadcrumb(data) {
        const [row] = data
        if (row) {
          const [path] = row

          if (path !== undefined) {
            let subpath = ''
            let localBreads = [
              {
                label: '',
                icon: 'home',
                ident: '',
              }
            ]

            localBreads.push(...path.split(':').map((part) => {
              subpath += `${subpath !== '' ? ':' : ''}${part}`
              const item = {
                label: part,
                ident: subpath,
              }
              return item
            }))

            this.breads = localBreads
          }
        }
      },

      onBreadClick(ev) {
        this.fetchData(ev.ident)
      },
      
      addRow(row, redraw) {
        /*
        this.datatable.datamanager.data.push(row)
        if (redraw) {
          this.datatable.refresh()
        }
        */
      },
      
      setRows(rows) {
        this.datatable.updateSettings({
          data: rows,
        })
      },
      
      setCellValueByElement(cellElement, value) {
      },
      getRow(cellElement) {
      },
      setSelectedCellsValue(value) {
      },
      async fetchData(path) {
        const server = await this.ctx.processManager.fetchServer(
		  this.link.processManagerUuid,
          this.link.instanceUuid,
          this.link.processUuid,
          'DMS',
          false,
          true,
          'dmsApi',
        )

        const url = this.ctx.processManager.getEndpoint(server)
        let get = []
        if (path === undefined) {
          get = [
            {
              path: '',
              query: {
                maxDepth: 0,
                regExPath: `.*:${this.link.path}`
              }
            }
          ]
        } else {
          get = [
            {
              path,
              query: {
                maxDepth: 1,
              }
            }
          ]
        }

        const subscription = {
          unsubscribe: [],
          subscribe: [],
        }

        if (path !== undefined) {
         if (this.response) {
            subscription.unsubscribe.push({
              path: this.response.payload.get[0].path,
              query: {
                maxDepth: 2,
              }
            })
          }
  
          subscription.subscribe.push({
            path,
            query: {
              maxDepth: 2,
            }
          })  

          this.dms.send({
            url,
            payload: {
              ...subscription
            },
          })
        }
        
        this.ctx.dms.post({
          url,
          payload: {
            get,
          }
        })
        .then((res) => {
          this.response = res

          if (res.payload && res.payload.get) {
            this.data = res.payload.get.map((node) => [
              node.path, 
              node.value, 
              node.type, 
              node.hasChild,
            ])
            if (res.payload.get.length) {
              switch (res.payload.get[0].type) {
                case 'bool':
                  this.valueType = 'checkbox'
                  break;
                case 'string':
                  this.valueType = 'text'
                  break;
                default:
                  this.valueType = 'number'
                  break;
              }
            }
            this.setRows(this.data)
            this.createBreadcrumb(this.data)
          }
        })
      }
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        components.page([56], this.init).append(() => [
          // any ui component node can be used...
          components.section().append(() => [
            components.form('pList').append(() => [
              components.element('NgPMManagerChooser').attrs({ class: 'col-6', modelValue: this.link.processManagerUuid, 'onUpdate:modelValue': (val) => this.link.processManagerUuid = val }),
              components.element('NgPMProcessChooser').attrs({ class: 'col-6', modelValue: this.link, 'onUpdate:modelValue': (val) => {
                this.link = val
                this.fetchData('')
              }}),
              components.input('path', 'text', this.link.path, (val) => this.link.path = val).attrs({ class: 'col-12' }),
            ]).on({
              onSubmit: () => this.fetchData(),
            }).attrs({
              submitLabel: 'read'
            }),
            components.form().append(() => [
              components.input('newValue', this.valueType, this.newValue, (val) => this.newValue = val).attrs({ class: 'col-12' }),
            ]).on({
              onSubmit: () => this.setSelectedCellsValue(this.newValue),
            }).attrs({
              submitLabel: 'set',
              class: 'q-mb-md'
            }),
            components.separator(),
            components.element('div').attrs({ class: 'q-mt-md full-width' }).append(() => [
              components.element('NgBreadcrumbs').attrs({
                items: this.breads,
                onClick: (ev) => this.onBreadClick(ev),
                separator: ':',
                clickable: true,
              }),
              components.element('div').attrs({ id: 'datatable', class: 'q-mt-md full-width' }),
            ]),
          ])
        ])
      ]
    },
  }
})