// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()
import useProcessManager from '/service/rest/v1/files/data/public/js/useProcessManager.js'
const processManager = useProcessManager()

window.ngui(() => {
  return {
    mixins: [
      processManager.mixin(),
    ],
    beforeCreate() {
      components.setParent(this)
      processManager.setParent(this)
    },

    
    // Listen on unmounted event...
    unmounted() {
      // logic
      this.reset()
    },
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        dialog: {
          add: false,
          remove: false,
        },
        selectable: false,
        url: {
          protocol: 'http:',
          hostname: '',
          port: 9018
        },
        formCopy: {},
        form: {
          slug: '',
          name: '',
          description: '',
          url: '',
          type: 'process-manager',
          redirect: true,
          authenticator: 'promos',
        },
        monitoring: {},
        protocolOptions: [
          {
            label: 'HTTP',
            value: 'http:',
          },
          {
            label: 'HTTPS',
            value: 'https:',
          },
        ],
        pingInterval: 0,
        loading: false,
      }
    },
      
    // Define methods for the ui...
    methods: {
      async init() {
        this.loading = true
        try {
          this.processManagers = await this.ctx.processManager.fetchManagers()
          this.pingStart()
          this.formCopy = components.getObjCopy(this.form)
          this.urlCopy = components.getObjCopy(this.url)  
        } catch (e) {
          // nothing to do
        }
        this.loading = false
      },
      
      reset() {
        this.pingStop()
      },
      
      fetchMonitoring() {
        const services = Object.keys(this.monitoring)
        services.forEach((service) => {
          if (this.monitoring[service] && this.monitoring[service].options.error) {
            delete this.monitoring[service]
          } 
        })
        this.ctx.httpClient.get(this.ctx.endpoints.getHttp(`/monitoring?timeout=1&type=process-manager`)).then((res) => {
          res.data.forEach((manager) => {
            this.monitoring[manager.service] = manager
          })
        })
      },

      pingStart() {
        this.pingStop()
        this.fetchMonitoring()
        this.pingInterval = window.setInterval(() => {
          this.fetchMonitoring()
        }, 30 * 1000)
      },

      pingStop() {
        window.clearInterval(this.pingInterval)
      },

      onDeleteSelected() {
        this.removeSelectedManagers().then(() => this.init())
      },

      getActionMenu(props) {
        return components.element('NgSection').attrs({ permissions: this.ctx.globals.getPermissions(['managersUpdate', 'managersDelete']) }).on({ onClick: this.withModifiers(() => {}, ['stop'])}).append(() => [
          components.menu().attrs({ 
            items: [
              {
                id: 'edit',
                label: 'edit',
                permissions: this.ctx.globals.getPermissions(['managersUpdate']),
                click: () => {
                  this.form = props.row.options
                  this.url = new URL(this.form.url)
                  this.dialog.add = true
                }
              },
              {
                id: 'delete',
                label: 'delete',
                permissions: this.ctx.globals.getPermissions(['managersDelete']),
                disable: props.row.options.slug === this.ctx.globals.names.localProcessManager,
                click: () => {
                  this.selectedManagers = [props.row]
                  this.dialog.remove = true
                }
              }
            ], 
          }),
        ])
      }
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        components.page([22], () => this.init(), () => this.reset()).append(() => [
          components.breadcrumbs(processManager.getPMBreadcrumbs()),
          
          components.bar('managers').attrs({
            id: 'managers-toolbar',
          }),
          
          components.section().append(() => [
            components.card().attrs({ flat: !this.ctx.platform.browser }).append(() => [
              components.table(
                'managers',
                [
                  {
                    i18nLabel: 'name',
                    field: (row) => row.options.name,
                    name: 'name',
                    sortable: true,
                  },
                  {
                    i18nLabel: 'description',
                    field: (row) => row.options.description,
                    name: 'description',
                    sortable: true,
                  },
                  {
                    i18nLabel: 'status',
                    name: 'monitoring',
                    sortable: true,
                  },
                  {
                    label: 'URL',
                    field: (row) => row.options.url,
                    name: 'url',
                    sortable: true,
                  },
                  {
                    label: 'ID',
                    field: (row) => row.service,
                    name: 'slug',
                  },
                  {
                    i18nLabel: 'action',
                    field: '',
                    name: 'action',
                  },
                ],
                this.processManagers,
              ).attrs({
                id: 'managers-table',
                selection: 'multiple',
                selected: this.selectedManagers,
                rowKey: 'service',
                grid: !this.ctx.platform.browser,
                flat: true,
                'onUpdate:selected': (val) => this.selectedManagers = val, 
                'onClick:row': (props) => components.router().push('/process-manager/index', { 
                  service: props.row.options.slug,
                }),   
                enableAddBtn: true,
                enableDeleteBtn: true,
                permissionsAdd: [21],
                permissionsDelete: [24],
                permissionsSelect: [24],
                loading: this.loading,
                'onClick:add': () => {
                  this.form = {}
                  this.dialog.add = true
                },
                'onConfirm:delete': () => {
                  this.onDeleteSelected()
                },
                maxHeight: this.ctx.platform.browser ? '75vh' : undefined,
            }).append('body-cell-name', (props) => [
              components.td().append(() => [
                components.flex().append(() => [
                  components.icon(
                    props.row.service === this.ctx.globals.names.localProcessManager ? 'star' : 'folder',
                    props.row.service === this.ctx.globals.names.localProcessManager ? 'primary' : 'grey',
                    'xs',
                  ),
                  components.text(props.value).attrs({ type: props.row.service === this.ctx.globals.names.localProcessManager ? 'bold' : '' })
                ])
              ])
            ]).append('body-cell-monitoring', (props) => [
              components.td().append(() => [
                processManager.getMonitoringBadge(this.monitoring[props.row.service])
              ])
            ]).append('body-cell-action', (props) => [
              components.td().append(() => [
                this.getActionMenu(props)
              ])
            ]).append('item', (props) => [
              components.tdItem(components.card().append(() => [
                components.renderIf(this.selectable, components.section().append(() => [
                  components.input('select', 'checkbox', props.selected, (val) => props.selected = val).attrs({
                    permissions: this.ctx.globals.getPermissions(['managersDelete']),
                  }),
                ])),
                components.section().append(() => [
                  components.item(props.row.options.name, props.row.options.description ? props.row.options.description : '').attrs({
                    clickable: true,
                    onClick: () => components.router().push('/process-manager/index', { 
                      service: props.row.options.slug,
                    }),
                  }).append('side', () => [
                    this.getActionMenu(props),
                  ]),
                ]),
              ]))
            ])
          ]),
          ]),
          
          components.dialogDelete(this.selectedManagers.map((item) => item.options.name).join(', '), this.dialog.remove, (val) => this.dialog.remove = val).on({
            onOk: () => {
              this.onDeleteSelected()
            },
          }),

          components.formDialog('processManager', this.dialog.add, (val) => this.dialog.add = val, 'public/forms/manager.js').formOn({
            onSubmit: () => {
              this.dialog.add = false
              this.init()
            },
            onCancel: () => this.dialog.add = false,
          }).formAttrs({
            slug: this.form ? this.form.slug : '',
          }).on({
            onHide: () => {
              this.form = {}
              this.fetchMonitoring()
            }
          })
        ])
      ]
    },
  }
})