// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    
    mixins: [
      window.processManager.mixin(),
    ],
    
    beforeCreate() {
      window.components.setParent(this)
      window.processManager.setParent(this)
    },
    
    // Listen on mounted event...
    mounted() {
      // logic
      this.isUnmounted = false
    },

    
    // Listen on unmounted event...
    unmounted() {
      // logic
      this.isUnmounted = true
      this.stopPoll()
    },

    // Watch primitive or object values for it state changes...
    watch: {
      // props
      'instanceStatus'(instance) {
        this.handleChartData(instance)
      }
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        isUnmounted: false,
        processManagerUuid: this.$route.query.service,
        instanceUuid: this.$route.query.instance,
        memOptions: {
          chart: {
            type: 'bar',
          },
          xaxis: {
            categories: [
              'Memory',
            ],
          },
          yaxis: {
            id: 'yaxis',
            title: {
              text: 'Mb',
            },
          },
          series: [],
        },
        cpuOptions: {
          chart: {
            type: 'bar',
          },
          xaxis: {
            categories: [
              'CPU',
            ],
          },
          yaxis: {
            id: 'yaxis',
            title: {
              text: '%',
            },
          },
          series: [],
        },
        memSeries: [],
        cpuSeries: [],
        intervalTime: 5000,
        intervalId: 0,
      }
    },

    
    // Define methods for the ui...
    methods: {
      // functions
      // functions
      async init() {
        this.stopPoll()
        try {
          this.instanceStatus = await this.ctx.processManager.fetchInstanceStatus(this.processManagerUuid, this.instanceUuid)
          this.instanceSettings = await this.ctx.processManager.fetchInstanceSettings(this.processManagerUuid, this.instanceUuid)
        } catch (e) {
          this.ctx.notify.notifyApiError(e)
          this.loading = false
        }
        this.pollStatus()
      },
  
      async pollStatus() {
        this.stopPoll()
        this.instanceStatus  = await this.ctx.processManager.fetchInstanceStatus(this.processManagerUuid, this.instanceUuid)
        this.startPollInterval()
      },
  
      startPollInterval() {
        this.stopPoll()
        if (this.isUnmounted) {
          return
        }
        this.intervalId = setInterval(async () => {
          this.instanceStatus = await this.ctx.processManager.fetchInstanceStatus(this.processManagerUuid, this.instanceUuid)
        }, this.intervalTime)
      },

      stopPoll() {
        clearInterval(this.intervalId)
      },

      getProcessData(process) {
        return {
          uuid: process.uuid,
          memUsage: Number((process.memUsage / (1 * 1000 * 1000)).toFixed(1)),
          cpuUsage: Number(process.cpuUsage.toFixed(1)),
        }
      },

      getGoalOfTopMemUsage(processData) {
        if (this.topMemUsage[processData.uuid] === undefined || this.topMemUsage[processData.uuid] < processData.memUsage) {
          this.topMemUsage[processData.uuid] = processData.memUsage
        }
        return {
          name: this.ctx.i18n.i18n.t('max'),
          value: this.topMemUsage[processData.uuid],
          strokeColor: '#775DD0'
        }
      },

      getGoalOfTopCpuUsage(processData) {
        if (this.topCpuUsage[processData.uuid] === undefined || this.topCpuUsage[processData.uuid] < processData.cpuUsage) {
          this.topCpuUsage[processData.uuid] = processData.cpuUsage
        }
        return {
          name: this.ctx.i18n.i18n.t('max'),
          value: this.topCpuUsage[processData.uuid],
          strokeColor: '#775DD0'
        }
      },

      handleChartData(instance) {   
        const cpuSeries = []
        const memSeries = []
        instance.processes.forEach((process) => {
          if (process.enabled) {
            const processData = this.getProcessData(process)
            memSeries.push({
              name: process.name,
              data:  [processData.memUsage],
            })

            cpuSeries.push({
              name: process.name,
              data:  [processData.cpuUsage],
            })
          }
        })

        this.memSeries = [...memSeries];
        this.cpuSeries = [...cpuSeries];
      }
    },
    
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        window.components.page([26], () => this.init(), () => this.stopPoll()).append(() => [
          window.components.breadcrumbs(window.processManager.getPMBreadcrumbs(this.processManagerUuid, this.instanceUuid, undefined, 'dashboard')),
          window.components.bar('instance'),
          window.components.section().attrs({ class: 'full-height' }).append(() => [
            window.components.grid().append(() => [
              window.components.cell().append(() => [
                window.components.card().append(() => [
                  window.components.section().append(() => [
                    window.components.chart('Memory [Mb]', this.memOptions, this.memSeries, 600),
                  ]),
                ]),  
              ]),
              window.components.cell().append(() => [
                window.components.card().append(() => [
                  window.components.section().append(() => [
                    window.components.chart('CPU [%]', this.cpuOptions, this.cpuSeries, 600),
                  ]),
                ]),
              ]),
            ]),
          ]),
        ]),
      ]
    },
  }
})