// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()

import useProcessManager from '/service/rest/v1/files/data/public/js/useProcessManager.js'
const processManager = useProcessManager()

window.ngui(() => {
  return {

    mixins: [
      processManager.mixin(),
    ],
  
    beforeCreate() {
      components.setParent(this)
      processManager.setParent(this)
    },

        // Listen on mounted event...
    // Listen on unmounted event...
    unmounted() {
      // logic
      this.stopPoll()
      this.isUnmounted = true
    },
    mounted() {
      // logic
      this.isUnmounted = false
      this.stopPoll()
    }, 
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // 
        loading: false,
        showChart: false,
        processManagerUuid: this.$route.query.service,
        instanceUuid: this.$route.query.instance,
        processUuid: this.$route.query.process,
        intervalId: null,
        intervalTime: 2500,
        isUnmounted: false,
        dialog: {
          remove: false,
          process: false,
        },
        hasModule: false,
        endpoint: '',
        chartOptions: {
          chart: {
            type: 'line',
          },
          xaxis: {
            type: 'datetime',
          },
          yaxis: [
            {
              id: 'cpu',
              min: 0,
              max: 100,
              startOnTick: true,
              endOnTick: true,
              labels: {
                style: {
                  color: 'red'
                }
              }
            },
            {
              id: 'memory',
              min: null,
              max: null,
              startOnTick: true,
              endOnTick: true,
              labels: {
                style: {
                  color: 'blue'
                }
              }
            }
          ],
          tooltip: {
              enabled: true,
              decimals: 2,
          },
        },
        timeSeries: [
          {
              name: 'CPU [%]',
              id: 'cpu',
              data: [],
              yAxis: 'cpu',
              color: 'red'
          },
          {
              name: 'Memory [MB]',
              id: 'memory',
              data: [],
              yAxis: 'memory',
              color: 'blue'
          }
        ],
        playTimeseries: true,
      }
    },
    
    watch: {
      processStatus(status) {
        this.handleTimeSeries(status)
      }
    },

    // Define methods for the ui...
    methods: {
      // functions
      async init() {
        try {
          this.loading = true
          this.loadingText = 'processManager'
          this.processManager = await this.ctx.processManager.fetchManager(this.processManagerUuid)
          this.loadingText = 'instance'
          this.instanceSettings = await this.ctx.processManager.fetchInstanceSettings(this.processManagerUuid, this.instanceUuid)
          this.loadingText = 'process'
          this.processStatus = await this.ctx.processManager.fetchProcessStatus(this.processManagerUuid, this.instanceUuid, this.processUuid)
          this.loadingText = 'process'
          this.processSettings = await this.ctx.processManager.fetchProcessSettings(this.processManagerUuid, this.instanceUuid, this.processUuid)
          this.hasModule = window.processManager.hasModule(this.processSettings.name)
          this.pollStatus()
        } catch (e) {
          this.ctx.notify.notifyApiError(e)
        }

        try {
          this.loadingText = 'server'
          this.processServer = await this.ctx.processManager.fetchServer(this.processManagerUuid, this.instanceUuid, this.processUuid)
          this.loadingText = 'endpoint'
          this.endpoint = this.ctx.processManager.getEndpoint(this.processServer) 
        } catch (e) {
          console.log(e)
        }
        this.loading = false
      },

      removeProcess() {
        this.stopPoll()
        this.ctx.processManager.deleteProcess(this.processManagerUuid, this.instanceUuid, this.processUuid)
          .then(() => {
            this.ctx.notify.notifyDeleted()
            this.$router.push({
              path: '/process-manager/instances/index',
              query: {
                service: this.$route.query.service,
                instance: this.instanceUuid,
              }
            })
          })
      },
  
      async pollStatus() {
        this.stopPoll()
        if (this.isUnmounted) {
          return
        }
        this.processStatus = await this.ctx.processManager.fetchProcessStatus(this.processManagerUuid, this.instanceUuid, this.processUuid)
        this.startPollInterval()
      },
  
      startPollInterval() {
        this.stopPoll()
        this.intervalId = setInterval(async () => {
          this.processStatus = await this.ctx.processManager.fetchProcessStatus(this.processManagerUuid, this.instanceUuid, this.processUuid)
        }, this.intervalTime)
      },

      stopPoll() {
        clearInterval(this.intervalId)
      },

      toConfiguration() {
        components.router().push('/process-manager/instances/processes/configuration', { 
          service: this.$route.query.service,
          instance: this.instanceUuid,
          process: this.processUuid,
          name: this.processSettings.name,
          bread: 'index',
        })
      },

      toEngineering() {
        components.router().push('/process-manager/instances/processes/process', {
          service: this.$route.query.service,
          instance: this.instanceUuid,
          process: this.processUuid,
          name: this.processSettings.name,
          bread: 'index',
        })
      },

      handleTimeSeries(status) {
        if (!this.playTimeseries) {
          return
        }
        const timestamp = new Date().valueOf()
        const { memUsage, cpuUsage } = status
        const cpuSerie = [timestamp, Number(cpuUsage.toFixed(1))]
        const memSerie = [timestamp, Number((memUsage / (1 * 1000 * 1000)).toFixed(0))]
        this.timeSeries[0].data.push(cpuSerie)
        this.timeSeries[1].data.push(memSerie)

        if (this.timeSeries[0].data.length > 600) {
          this.timeSeries[0].data.shift()
        }

        if (this.timeSeries[1].data.length > 600) {
          this.timeSeries[1].data.shift()
        }
        this.timeSeries = [... this.timeSeries];
        
      },

      resetTimeserie() {
        this.timeSeries[0].data = []
        this.timeSeries[1].data = []
      },

      handlePlayTimeseries() {
        this.playTimeseries = !this.playTimeseries
        if (this.playTimeseries) {
          this.resetTimeserie()
        }
      }
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        components.page([30], () => this.init(), () => this.stopPoll()).append(() => [

          components.breadcrumbs(processManager.getPMBreadcrumbs(this.processManagerUuid, this.instanceUuid, this.processUuid)),

          // Bar
          components.bar('process').createMenu('file', [
            {
              id: 'edit',
              label: 'edit',
              icon: 'edit',
              permissions: this.ctx.globals.getPermissions(['processesUpdate']),
              click: () => {
                this.dialog.process = true
                this.stopPoll()
              }
            },
            {
              id: 'delete',
              label: 'delete',
              icon: 'delete',
              permissions: this.ctx.globals.getPermissions(['processesDelete']),
              click: () => {
                this.dialog.remove = true
              }
            },
          ]).createMenu('view', [
            {
              id: 'logs',
              label: 'logs',
              icon: 'description',
              permissions: this.ctx.globals.getPermissions(['changelogRead']),
              click: () => {
                window.processManager.toLogs(this.processManagerUuid, this.instanceUuid, this.processUuid)
              }
            },

            {
              id: 'status',
              label: 'status',
              icon: 'checklist_rtl',
              disable: !this.endpoint,
              permissions: this.ctx.globals.getPermissions(['processesUpdate']),
              click: () => {
                window.processManager.toStatus(this.processManagerUuid, this.instanceUuid, this.processUuid)
              }
            },

            {
              id: 'module',
              label: 'module',
              icon: 'web',
              disable: !this.hasModule,
              click: () => {
                window.processManager.toModule('DMS', this.processManagerUuid, this.instanceUuid, this.processUuid)
              }
            },
          ]).attrs({
            id: 'process-toolbar',
          }),

          components.section().append(() => [
            components.grid().append(() => [
              components.cell(4).append(() => [
                components.list(`${this.ctx.i18n.i18n.t('process')}: ${this.processSettings.name}`).attrs({ 
                  dense: false, 
                  id: 'process-list',
                  flat: false,
                }).append(() => [
                  components.item().append('section', () => [
                    processManager.getActionButtons(this.processStatus, (ident) => {
                      this.ctx.processManager.putProcessAction(this.processManagerUuid, this.instanceUuid, this.processUuid, ident)
                    }).attrs({ 
                      id: 'process-action',
                      permissions: this.ctx.globals.getPermissions(['processesUpdate']),
                    }),
                  ]),
                  components.item().append('section', () => [
                    processManager.getStatusComponents(this.processStatus),
                  ]).append('side', () => [
                    components.menu().attrs({ items: processManager.getActionMenu(this.processStatus, {
                      update: [31],
                      delete: [32],
                    }), enableGlobalClick: true, disable: this.processStatus.name.includes('ProcessManager') }).on({ 
                      'onClick:menu': (ident) => {
                        if (ident === 'edit') {
                          this.dialog.process = true
                        } else if (ident === 'delete') {
                          this.dialog.remove = true
                        } else {
                          this.ctx.processManager.putProcessAction(this.processManagerUuid, this.instanceUuid, this.processUuid, ident)
                        }
                      },
                    })
                  ]),
                  components.input('enabled', 'checkbox', this.processSettings.enabled, (val) => {
                    this.processSettings.enabled = val
                    this.ctx.processManager.putProcessSettings(this.processManagerUuid, this.instanceUuid, this.processUuid, this.processSettings)
                  }).attrs({
                    hint: 'enableAndAutoStart',
                    permissions: this.ctx.globals.getPermissions(['processesUpdate']),
                  }),
                  components.item('manager', this.processManager.name),
                  components.item('project', this.instanceSettings.name),
                  components.item('name', this.processStatus.name),
                  components.item('description', this.processStatus.description),
                  components.item('uuid', this.processStatus.uuid).attrs({ dense: false }),
                  components.item('server', this.ctx.processManager.getEndpoint(this.processServer) || 'noData'),
    
                  components.renderIf(this.processStatus.tsLastStart, components.item('tsLastStart', components.getDateString(this.processStatus.tsLastStart))),
                  components.renderIf(this.processStatus.tsLastStop, components.item('tsLastStop', components.getDateString(this.processStatus.tsLastStop))),
                  components.renderIf(
                    this.processStatus.lastErrorText && this.processStatus.tsLastError > this.processStatus.tsLastStart,
                    components.item('lastErrorText', this.processStatus.lastErrorText).append('side', () => [
                      components.icon(this.processStatus.lastErrorText ? 'notifications_active' : 'notifications_active').attrs({
                        color: this.processStatus.lastErrorText ? 'negative' : ''
                      })
                    ]),
                  ),
                  components.renderIf(
                    this.processStatus.tsLastError && this.processStatus.tsLastError > this.processStatus.tsLastStart,
                    components.item('tsLastError', components.getDateString(this.processStatus.tsLastError))
                  ),
                  components.expandItem('performance').attrs({
                    onClick: () => this.showChart = !this.showChart,
                  }).append(() => [
                    components.renderIf(this.showChart, components.chart('', this.chartOptions, this.timeSeries, 250)),
                    components.button('', this.playTimeseries ? 'stop' : 'play_arrow').on({
                      onClick: () => this.handlePlayTimeseries()
                    }).attrs({
                      color: this.playTimeseries ? 'negative' : 'positive',
                      tooltip: 'chart',
                      flat: true,
                      dense: true,
                    }),
                  ]),           
                ])
              ]),

              components.cell(8).append(() => [
                components.list('modules').attrs({ 
                  dense: false, 
                  id: 'module-list',
                  flat: false,
                  loading: this.loading,
                  loadingLabel: this.loadingText,
                }).append(() => [
                  components.item('management', 'managementDescription').attrs({
                    clickable: true,
                    id: 'process-engineering',
                    permissions: this.ctx.globals.getPermissions(['processesUpdate']),
                  }).on({
                    onClick: () => this.toEngineering(),
                  }).append('preside', () => [
                    components.icon('mdi-account-hard-hat-outline'),
                  ]), 
                  components.item('module', 'moduleDescription').attrs({
                    clickable: true,
                    disable: this.processStatus.status !== 2 || !this.hasModule,
                    id: 'process-module',
                  }).on({
                    onClick: () => window.processManager.toModule(this.processSettings.name, this.$route.query.service, this.instanceUuid, this.processUuid),
                  }).append('preside', () => [
                    components.icon('web'),
                  ]),
                  components.item('configuration', 'configurationDescription').attrs({
                    clickable: true,
                    id: 'process-configuration',
                    permissions: this.ctx.globals.getPermissions(['processesUpdate']),
                  }).on({
                    onClick: () => this.toConfiguration(),
                  }).append('preside', () => [
                    components.icon('settings'),
                  ]),
                  components.item('status', 'statusDescription').attrs({
                    clickable: true,
                    disable: this.processStatus.status !== 2 || !this.endpoint,
                    id: 'process-status',
                    permissions: this.ctx.globals.getPermissions(['processesUpdate']),
                  }).on({
                    onClick: () => window.processManager.toStatus(this.$route.query.service, this.instanceUuid, this.processUuid),
                  }).append('preside', () => [
                    components.icon('checklist_rtl'),
                  ]),
                  components.item('logs', 'logDescription').attrs({
                    clickable: true,
                    id: 'process-logs',
                    permissions: this.ctx.globals.getPermissions(['changelogRead']),
                  }).on({
                    onClick: () => window.processManager.toLogs(this.$route.query.service, this.instanceUuid, this.processUuid),
                  }).append('preside', () => [
                    components.icon('description'),
                  ]), 

                  
                  components.renderIf(!this.loading && this.processStatus.status !== 2, components.item().append('section', () => [
                    window.processManager.getStatusBanners(this.processStatus),
                  ])),
                ]),
          
              ]),

            ]),
              
          ]),

          // Delete
          components.dialogDelete(this.processStatus.name, this.dialog.remove, (val) => this.dialog.remove = val).on({
            onOk: () => this.removeProcess(),
          }), 

          // Edit
          components.formDialog('process', this.dialog.process, (val) => this.dialog.process = val, 'public/forms/process.js').formOn({
            onSubmit: () => {
              this.dialog.process = false
            },
            onCancel: () => {
              this.dialog.process = false
            },
          }).formAttrs({
            instanceUuid: this.instanceUuid,
            processUuid: this.processUuid,
          }).on({
            onHide: () => this.init(),
            onShow: () => this.stopPoll()
          }),
        ]),
      ]
    },
  }
})