// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()

import useProcessManager from '/service/rest/v1/files/data/public/js/useProcessManager.js'
const processManager = useProcessManager()

window.ngui(() => {
  return {

    mixins: [
      processManager.mixin(),
    ],
  
    beforeCreate() {
      components.setParent(this)
      processManager.setParent(this)
    },

    mounted() {
      this.isUnmounted = false
    },

    // Listen on unmounted event...
    unmounted() {
      // logic
      clearTimeout(this.reconnectId)
      if (this.socket) {
        this.socket.close(1000)
      }
      this.isUnmounted = true
    },
     
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        isUnmounted: false,
        reconnectId: 0,
        processName: this.$route.query.name,
        processManagerUuid: this.$route.query.service,
        instanceUuid: this.$route.query.instance,
        processUuid: this.$route.query.process,
        logId: this.$route.query.logId ? Number(this.$route.query.logId) : 1,
        isConnected: true,
        loading: false,
        loadingText: 'loading',
        socket: {},
        enumLogs: [],
        logName: '',
        columns: [],
        rows: [],
        form: {
          maxLines: 500,
          run: true,
        },
        dialog: {
          log: false,
          settings: false,
        },
        selectedLog: {},
        pagination: components.createDefaultPagination(100),
        hasModule: false,
      }
    },
  
      
    // Define methods for the ui...
    methods: {
      // functions 
      async init() {
        this.ctx.endpoints.toService([this.processManagerUuid], true, false)
        try {
          this.loading = true
          this.loadingText = 'processManager'
          this.processManager = await this.ctx.processManager.fetchManager(this.processManagerUuid)    
          this.loadingText = 'instance'
          this.instanceSettings = await this.ctx.processManager.fetchInstanceSettings(this.processManagerUuid, this.instanceUuid)
          this.loadingText = 'process'
          this.processStatus = await this.ctx.processManager.fetchProcessStatus(this.processManagerUuid, this.instanceUuid, this.processUuid)
          this.loadingText = 'process'
          this.processSettings = await this.ctx.processManager.fetchProcessSettings(this.processManagerUuid, this.instanceUuid, this.processUuid)
          this.hasModule = window.processManager.hasModule(this.processSettings.name)
          this.loading = false
          this.loadingText = 'connecting'
        } catch (e) {
          this.ctx.notify.notifyApiError(e)
          this.loading = false
        }

        this.initSocket()

      },
  
      sendEnumLogs() {
        this.socket.send({command:'enumLogs'})
      },
  
      sendSubscribeLog() {
        this.socket.send({command:'subscribeLog', logId: this.logId, maxLines: this.form.maxLines})
      },
  
      onClickEnumLog(name, id) {
        this.logId = id
        this.logName = name
        this.sendSubscribeLog()
        this.rows = []
        this.ctx.routing.replace({ query: { ...this.ctx.routing.route.query, logId: id } })
      },
  
      handleRows(lines = []) {
        lines = lines.filter((log) => log.msg !== '').reverse()
        this.rows = [ ...lines, ...this.rows ].slice(0, this.form.maxLines)
      },

      handleColumns(lines = []) {
        let colsByName = {}
        lines.forEach((line) => {
          const cols = components.createTableColsByObject(line)
          cols.forEach((col) => {
            col.style = 'max-width: 250px;'
            col.classes = 'text-ellipsis ellipsis'
            colsByName[col.name] = col;
          })
        })

        this.columns = Object.values(colsByName);
      },
  
      messageHandler(data) {
        switch (data.event) {
          case 'enumLogs':
            this.enumLogs = data.logs.map((log, idx) => ({ id: idx + 1, label: log, click: () => this.onClickEnumLog(log, idx + 1) }))
            if (this.enumLogs.length) {
              this.logId = this.ctx.routing.route.query.logId === undefined ? this.enumLogs[0].id : this.logId
              this.logName = this.enumLogs[this.logId - 1] ? this.enumLogs[this.logId - 1].label : 'Log'
              this.sendSubscribeLog()
            }
            break
          case 'logLines':
            if (this.columns.length <= 0 && data.lines.length > 0) {
              this.handleColumns(data.lines)
            }
            this.handleRows(data.lines)
            break
          case 'error':
            this.ctx.notify.plugin.error(data.message)
            this.rows = []
            break
        }
      },
      
      initSocket() {
        if (this.isUnmounted) {
          return
        }
        const { Socket } = this.ctx
        this.loading = true
        clearTimeout(this.reconnectId)
        this.socket = new Socket()
        this.socket.setAuth(true)
        this.socket.onConnected(() => {
          this.loading = false
          this.sendEnumLogs()
        })

        this.socket.onClosed((ev) => {
          this.loading = false
          if (ev.code !== 1000 && !this.isUnmounted) {
            this.isConnected = false
            this.reconnectId = setTimeout(() => this.initSocket(), 1000)
          }
        })

        this.socket.onError((err) => {
          this.isConnected = false
          this.loading = false
          this.ctx.notify.plugin.error(err.message)
        })

        this.socket.onMessage((data = { type: String, data: Object}) => {
          this.isConnected = true
          try {
            this.messageHandler(data.data)
          } catch (e) {
            console.error(e)
          }
        })

        this.socket.connect(this.ctx.endpoints.getWs(`/service/${this.ctx.endpoints.getLocalService()}/v1/logs/${this.processUuid}`))
      },
  
      getBadgeColor(level) {
        let color = 'grey'
        
        switch (level) {
          case 'error':
            color = 'red'
            break
          case 'info':
            color = 'blue-4'
            break
          case 'debug':
            color = 'dark'
            break
          case 'warn':
            color = 'orange-4'
            break
          case 'panic':
            color = 'negative'
            break
        }
  
        return color
      },
  
      getMaxLineOptions() {
        let base = 0
        return Array.from(Array(10).keys()).map(() => {
          base += 100
          return { value: base, label: base.toString() }
        })
      },
    
      updateMaxLines(val) {
        this.form.maxLines = val
        this.sendSubscribeLog()
      },
  
      updateRun(val) {
        this.form.run = val
  
        if (this.form.run) {
          this.initSocket()
        } else {
          this.socket.close(1000)
        }
      },
  
      onLoadLog(log) {
        this.selectedLog = log
        this.dialog.log = true
      },

      getDialogValueElement(value) {
        let val = value
        let component = components.element('div').attrs({ style: 'overflow: auto;' }).append(() => [
          val,
        ])
        
        if (typeof value === 'string' && value.startsWith('{')) {
          try {
            val = JSON.stringify(JSON.parse(value), null, 2)
            component = components.element('NgFormInput').attrs({ type: 'editor', lang: 'json', modelValue: val })
          } catch (e) {
            console.error(e)
          }
        }

        return component
      }
      
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
          components.page([46], () => this.init()).append(() => [
            components.renderIf(!this.$attrs.disableBread, components.breadcrumbs(processManager.getPMBreadcrumbs(this.processManagerUuid, this.instanceUuid, this.processUuid, 'logs'))),
            
            components.bar('process')
              .createMenu('file', this.enumLogs)
              .createMenu('view', [
              {
                id: 'logs',
                label: 'logs',
                icon: 'description',
                disable: true,
                click: () => {
                  components.router().push('/process-manager/instances/processes/logs/index', { 
                    service: this.$route.query.service,
                    instance: this.instanceUuid,
                    process: this.processUuid,
                    processName: this.$route.query.name,
                  })
                }
              },
  
              {
                id: 'status',
                label: 'status',
                icon: 'checklist',
                permissions: this.ctx.globals.getPermissions(['processesUpdate']),
                click: () => {
                  components.router().push('/process-manager/instances/processes/status/index', { 
                    service: this.$route.query.service,
                    instance: this.instanceUuid,
                    process: this.processUuid,
                    processName: this.$route.query.name,
                  })
                }
              },
  
              {
                id: 'module',
                label: 'module',
                icon: 'view_module',
                disable: !this.hasModule,
                click: () => {
                  components.router().push(`/modules/${this.processSettings.name}/index`, { 
                    service: this.$route.query.service,
                    instance: this.instanceUuid,
                    process: this.processUuid,
                    processName: this.$route.query.name,
                  })
                }
              },
            ])
            .createMenu('filter', [
              {
                id: 'view',
                label: 'view',
                click: () => this.dialog.settings = true,
              }
            ]),
  
            components.section().append(() => [
              components.renderIf(!this.isConnected && this.form.run, components.bannerNegative(this.ctx.i18n.i18n.t('notConnected'))),

              components.table(this.logName, this.columns, this.rows).pagination(this.pagination, (val) => this.pagination = val).on({
                  'onClick:row': (props) => this.onLoadLog(props.row) 
                })
                .attrs({ 
                  configIdent: 'pm-log',
                  maxHeight: '80vh',
                  flat: false,
                  grid: !this.ctx.platform.browser,
                  loading: this.loading,
                  loadingLabel: this.loadingText,
                }).append('body-cell-msg', (props) => [
                components.td(props).append(() => [
                  props.value
                ]).attrs({ 
                  class: 'ellipsis', 
                  style: { 
                    maxWidth: this.ctx.platform.mobile ? '250px' : '500px' 
                  } 
                })
              ]).append('body-cell-timestamp', (props) => [
                components.td(props).append(() => [
                  new Date(props.value).toLocaleString()
                ])
              ]).append('body-cell-level', (props) => [
                components.td(props).append(() => [
                  components.badge(props.value).attrs({ color: this.getBadgeColor(props.value) })
                ])
              ]).append('item', (props) => [
                  components.tdItem(components.card().append(() => [
                    components.list().append(() => [
                      ...this.columns.map((col) => {
                        let element = null
                        switch (col.name) {
                          case 'timestamp':
                            element = components.item(col.i18nLabel, new Date(props.row[col.name]).toLocaleString())
                            break
                          case 'level':
                            element = components.item(col.i18nLabel).append('side', () => [
                              components.badge(props.row[col.name]).attrs({ color: this.getBadgeColor(props.row[col.name]) })
                            ])
                            break
                          default:
                            element = components.item(col.i18nLabel, props.row[col.name])
                            break
                        }

                        element.attrs({
                          clickable: true,
                          icon: '',
                          onClick: () => this.onLoadLog(props.row),
                        })
                        return element
                      })
                    ])
                  ]))
                ]),
            ]),
            
            components.dialog('log', this.dialog.log, (val) => this.dialog.log = val).append(() => [
              components.list().append(() => [
                components.item('level', this.selectedLog.level),
                components.item('timestamp', new Date(this.selectedLog.timestamp).toLocaleString()),
              ]),
  
              components.section().append(() => [
                components.list().attrs({ bordered: true }).append(() => [
                  ...Object.keys(this.selectedLog).map((log) => components.item(log).append('subtitle', () => [
                     this.getDialogValueElement(this.selectedLog[log]),
                  ]))
                ]),
              ]),
            ]),  

            // Settings dialog
            components.dialog('view', this.dialog.settings, (val) => this.dialog.settings = val).append(() => [
              components.list().append(() => [
                components.item().append(() => [
                 components.groups().append(() => [
                    components.group('filter').append(() => [
                      components.input('showLines', 'select', this.form.maxLines, (val) => this.updateMaxLines(val)).attrs({ options: this.getMaxLineOptions() }),
                      components.input('update', 'toggle', this.form.run, (val) => this.updateRun(val)),
                    ]),
                  ])         
                ]),
              ]),
            ])
          ]),
      ]
    },
  }
})