// Call ngui function to create a ui config.
import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()

import useProcessManager from '/service/rest/v1/files/data/public/js/useProcessManager.js'
const processManager = useProcessManager()

const StatusTable = window.nguiAsyncImport('public/ui/main/process-manager/instances/processes/status/index.js')
const LogTable = window.nguiAsyncImport('public/ui/main/process-manager/instances/processes/logs/index.js')
const DmsTable = window.nguiAsyncImport('public/ui/main/modules/DMS/index.js')

window.ngui(() => {
  return {

    mixins: [
      processManager.mixin(),
    ],
  
    beforeCreate() {
      components.setParent(this)
      processManager.setParent(this)
    },

    // Listen on mounted event...
    mounted() {
      // logic
      this.isUnmounted = false
    },
    // Listen on unmounted event...
    unmounted() {
      // logic
      this.isUnmounted = true
      this.stopPoll()
    },

    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        processName: this.$route.query.name,
        processManagerUuid: this.$route.query.service,
        instanceUuid: this.$route.query.instance,
        processUuid: this.$route.query.process,
        endpoint: '',
        config: null,
        loading: true,
        intervalId: 0,
        isUnmounted: false,
        intervalTime: 2000,
        statusCode: 0,
        splitter: {
          name: 'window1',
          horizontal: false,
          panes: [
            {
              slot: 'top-left',
            },
            {
              slot: 'top-right',
            },
            {
              slot: 'bottom',
              horizontal: false,
            },
          ],
        },
        hasDms: false,
        dmsProcessUuid: '',
        dialog: {
          dms: false,
          log: false,
        }
      }
    },

    // Define methods for the ui...
    methods: {
      fetchConfig(pathname) {
        return new Promise((resolve, reject) => {
          this.ctx.filedb.read(pathname, true)
          .then((res) => {
            this.config = res.data.data
            resolve(this.config)
          })
          .catch((err) => {
            reject(err)
          })
        })
      },

      // functions
      async init() {
        this.loading = true
        try {
          this.processManager = await this.ctx.processManager.fetchManager(this.processManagerUuid)
          this.instanceSettings = await this.ctx.processManager.fetchInstanceSettings(this.processManagerUuid, this.instanceUuid)
          this.processSettings = await this.ctx.processManager.fetchProcessSettings(this.processManagerUuid, this.instanceUuid, this.processUuid)
          this.processStatus = await this.ctx.processManager.fetchProcessStatus(this.processManagerUuid, this.instanceUuid, this.processUuid)
          this.statusCode = this.processStatus.status
          const dms = this.instanceSettings.processes.find((proc) => proc.name === 'DMS')
          if (dms) {
            this.dmsProcessUuid = dms.uuid
          }
          
        } catch (e) {
          this.ctx.notify.notifyApiError(e)          
        }

        try {
          this.processServer = await this.ctx.processManager.fetchServer(this.processManagerUuid, this.instanceUuid, this.processUuid)
          this.endpoint = this.ctx.processManager.getEndpoint(this.processServer) 
        } catch (e) {
          console.log(e)
        }

        this.fetchConfig(`/cfg/configurations/${this.processName}.yaml`)
          .then(() => {
            return this.config
          })
          .catch(() => {
            return this.fetchConfig('cfg/configurations/templates/settingsUnknown.yaml')
          })
          .finally(() => {
            this.loading = false
          })

          this.pollStatus();
      },

      async pollStatus() {
        this.stopPoll()
        if (this.isUnmounted) {
          return
        }
        this.processStatus = await this.ctx.processManager.fetchProcessStatus(this.processManagerUuid, this.instanceUuid, this.processUuid)
        this.startPollInterval()
      },
  
      startPollInterval() {
        this.stopPoll()
        this.intervalId = setInterval(async () => {
          this.processStatus = await this.ctx.processManager.fetchProcessStatus(this.processManagerUuid, this.instanceUuid, this.processUuid)
          if (this.statusCode !== this.processStatus.status) {
            this.statusCode = this.processStatus.status
          }
        }, this.intervalTime)
      },

      stopPoll() {
        clearInterval(this.intervalId)
      },

      getProcessActionCard() {
        // action button
        return components.element('div').append(() => [
            components.card().attrs({ flat: false, title: `${this.ctx.i18n.i18n.t('process')}: ${this.processSettings.name}`, class: 'full-height' }).append(() => [
              components.section().attrs({style: { overflow: 'auto', height: '73vh' }}).append(() => [
                   components.list().attrs({ 
                      dense: false, 
                      id: 'process-list',
                      flat: true,
                    }).append(() => [
                      components.item().append('section', () => [
                        processManager.getActionButtons(this.processStatus, (ident) => {
                          this.ctx.processManager.putProcessAction(this.processManagerUuid, this.instanceUuid, this.processUuid, ident)
                        }).attrs({ 
                          id: 'process-action',
                          permissions: this.ctx.globals.getPermissions(['processesUpdate']),
                        }),
                      ]),
                      components.item().append('section', () => [
                        processManager.getStatusComponents(this.processStatus),
                      ]).append('side', () => [
                        components.menu().attrs({ items: processManager.getActionMenu(this.processStatus, {
                          update: [31],
                          delete: [32],
                        }), enableGlobalClick: true, disable: this.processStatus.name.includes('ProcessManager') }).on({ 
                          'onClick:menu': (ident) => {
                            if (ident === 'edit') {
                              this.dialog.process = true
                            } else if (ident === 'delete') {
                              this.dialog.remove = true
                            } else {
                              this.ctx.processManager.putProcessAction(this.processManagerUuid, this.instanceUuid, this.processUuid, ident)
                            }
                          },
                        })
                      ]),
                      components.input('enabled', 'checkbox', this.processSettings.enabled, (val) => {
                        this.processSettings.enabled = val
                        this.ctx.processManager.putProcessSettings(this.processManagerUuid, this.instanceUuid, this.processUuid, this.processSettings)
                      }).attrs({
                        hint: 'enableAndAutoStart',
                        permissions: this.ctx.globals.getPermissions(['processesUpdate']),
                      }),
                      components.item('manager', this.processManager.name),
                      components.item('project', this.instanceSettings.name),
                      components.item('name', this.processStatus.name),
                      components.item('description', this.processStatus.description),
                      components.item('uuid', this.processStatus.uuid).attrs({ dense: false }),
                      components.item('server', this.endpoint || 'noData'),
        
                      components.renderIf(this.processStatus.tsLastStart, components.item('tsLastStart', components.getDateString(this.processStatus.tsLastStart))),
                      components.renderIf(this.processStatus.tsLastStop, components.item('tsLastStop', components.getDateString(this.processStatus.tsLastStop))),
                      components.renderIf(
                        this.processStatus.lastErrorText && this.processStatus.tsLastError > this.processStatus.tsLastStart,
                        components.item('lastErrorText', this.processStatus.lastErrorText).append('side', () => [
                          components.icon(this.processStatus.lastErrorText ? 'notifications_active' : 'notifications_active').attrs({
                            color: this.processStatus.lastErrorText ? 'negative' : ''
                          })
                        ]),
                      ),
                      components.renderIf(
                        this.processStatus.tsLastError && this.processStatus.tsLastError > this.processStatus.tsLastStart,
                        components.item('tsLastError', components.getDateString(this.processStatus.tsLastError))
                      ),     
                    ])
              ]),
            ])
          ])
          .attrs({
            class: 'col-12 col-md-3',
          })
      },

      getConfigCard() {
        return components.element('div').append(() => [
          components.card().attrs({ flat: false, title: 'configuration', class: 'full-height' }).append(() => [
            components.section().attrs({style: { overflow: 'auto', height: '73vh' }}).append(() => [
              components.processConfig(undefined, this.processManagerUuid, this.instanceUuid, this.processUuid, this.config).attrs({})
            ]),
          ])
        ])
        .attrs({
          class: 'col-12 col-md-3',
        })
      },

      getStatusCard() {
        return components.card().attrs({ flat: false, title: 'status', class: 'full-height'  }).append(() => [
          components.section().attrs({style: { overflow: 'auto', height: '73vh' }}).append(() => [
            components.element(StatusTable).attrs({
              name: this.processName,
              processManager: this.processManagerUuid,
              instance: this.instanceUuid,
              process: this.processUuid,
              statusIds: [''],
              flat: true,
              status: this.statusCode,
              maxHeight: '69vh',
              pagination: 100,
            })
          ]),
        ])
      },

      getRootElement() {
        return components.page([31], () => this.init())
      }
      
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        this.getRootElement().append(() => [
          
          components.breadcrumbs(processManager.getPMBreadcrumbs(this.processManagerUuid, this.instanceUuid, this.processUuid, 'management')),
          
          components.bar('process').createMenu('view', [
                {
                  id: 'logs',
                  label: 'logs',
                  icon: 'description',
                  permissions: this.ctx.globals.getPermissions(['changelogRead']),
                  click: () => {
                    this.dialog.log = true
                  }
                },
  
                {
                  id: 'DMS',
                  label: 'DMS',
                  icon: 'mdi-database',
                  disable: this.hasDms,
                  click: () => {
                    this.dialog.dms = true
                  }
                },
            ]),
          
          components.section().append(() => [
            components.card().attrs({flat: false}).append(() => [
              components.renderIf(!this.loading, components.element('NgSplitter').attrs({
                  modelValue: this.splitter,
                  'onUpdate:modelValue': (val) => this.splitter = val,
                  height: '75vh',
                }).append('top-left', () => [
                  this.getProcessActionCard()
                ]).append('top-right', () => [
                  this.getConfigCard()
                ]).append('bottom', () => [
                  this.getStatusCard()
                ])),
  
              components.dialog('log', this.dialog.log, (val) => this.dialog.log = val).append(() => [
                components.element(LogTable).attrs({
                  disableBread: true,
                }),
              ]),
  
              components.dialog('dms', this.dialog.dms, (val) => this.dialog.dms = val).append(() => [
                components.element(DmsTable).attrs({
                  processUuid: this.dmsProcessUuid,
                  disableBread: true,
                }),
              ])
            ]),
          ]),
        ])
      ]
    },
  }
})