import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()

// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    
    // Listen on mounted event...
    mounted() {
      // logic
      components.setParent(this)
    },
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        endpoint: '/discovery',
        method: 'get',
        body: '',
        response: null,
        err: null,
      }
    },

    
    // Define methods for the ui...
    methods: {
      // functions
      submit() {
        this.err = ''
        this.response = ''
        this.ctx.httpClient[this.method](this.endpoint, this.body)
          .then((res) => {
            this.response = JSON.stringify(res, null, 2)
          })
          .catch((err) => {
            console.log(err)
            this.response = err.response ? JSON.stringify(err.response.data, null, 2) : ''
            this.err = err.message
          })
      }
    },
      
    // Define elements within the ui configuration...
    nodes() {
      return [
        components.page([56]).append(() => [
          // any ui component node can be used...
          components.section().append(() => [        
            components.form('Request').append(() => [
              components.input('URL', 'text', this.endpoint, (val) => this.endpoint = val),
              components.input('Method', 'select', this.method, (val) => this.method = val).attrs({
                options: [
                  {
                    label: 'GET',
                    value: 'get',
                  },
                  {
                    label: 'POST',
                    value: 'post',
                  },
                  {
                    label: 'PUT',
                    value: 'put',
                  },
                  {
                    label: 'DELETE',
                    value: 'delete',
                  }
                ]
              }),
              components.renderIf(
                this.method === 'post' || this.method === 'put', 
                components.input('Body', 'editor', this.body, (val) => this.body = val).attrs({ lang: 'json'})
              )
            ]).on({
              onSubmit: () => this.submit(),
            }).attrs({
              submitLabel: 'send',
            }),
          ]),
          components.section().append(() => [  
            components.card('Response').append(() => [
              components.section().append(() => [
                components.renderIf(this.err, components.bannerNegative(this.err)),
                components.renderIf(
                  this.response, 
                  components.input('Body', 'editor', this.response, (val) => this.response = val).attrs({lang: 'json'})
                ),
              ])
            ])
          ])
        ])
      ]
    },
  }
})