import useComponent from '/service/rest/v1/files/data/public/js/useComponent.js'
const components = useComponent()

// Call ngui function to create a ui config.
window.ngui(() => {
  return {
        
  // Listen on mounted event...
  mounted() {
    // logic
    components.setParent(this)
  },
    
  // The returned object can be accessed with this within any method...
  data() {
    return {
      // VAR
      columns: [
          {
              name: 'name',
              label: 'Name',
              sortable: true,
              align: 'left',
              field: (row) => row.options.name,
          },
          {
              name: 'slug',
              label: 'Slug',
              align: 'left',
              field: (row) => row.options.slug,
          },
          {
              name: 'type',
              label: 'Type',
              sortable: true,
              align: 'left',
              field: (row) => row.options.type,
          },
          {
              name: 'url',
              label: 'URL',
              sortable: true,
              align: 'left',
              field: (row) => row.options.url,
          },
          {
              name: 'secure',
              label: 'Secure',
              sortable: true,
              align: 'left',
              field: (row) => row.options.secure ? '🔑' : '',
          },
      ],
      formSchema: {
        title: 'Service',
        type: 'object',
        properties: {
          service: {
            type: 'string',
            title: 'service',
            readOnly: true,
          },
          options: {
            type: 'object',
            title: 'options',
            properties: {
              name: {
                  type: 'string',
                  title: 'Name',
              },
              slug: {
                  type: 'uuid',
                  title: 'Slug',
              },
              url: {
                  type: 'string',
                  title: 'URL',
              },
              rootPath: {
                  type: 'string',
                  title: 'Root-Path',
              },
              secure: {
                  type: 'boolean',
                  title: 'secure',
              },
              type: {
                  type: 'string',
                  title: 'Type',
              },
              redirect: {
                  type: 'boolean',
                  title: 'Redirect',
              },
            }
          }
      },
    },
    }
  },

  // Define methods for the ui...
  methods: {
    // functions
    updateSlugUuuid() {
      this.formSchema.properties.options.properties.slug.const = this.ctx.uuidv4();
    }
  },
    
  // Define elements within the ui configuration...
  nodes() {
    return [
      // any ui component node can be used...
      components.page([56]).append(() => [
        components.bar(),
        components.section().append(() => [
          components.element('NgApiTable').attrs({
            title: 'Services',
            columns: this.columns,
            formSchema: this.formSchema,
          }).on({
            onApiRead: () => this.ctx.httpClient.get(this.ctx.endpoints.getHttp('/discovery')),
            onRead: (res) => {
              return res.data
            },
            onApiCreate: (rows, data) => this.ctx.httpClient.post(this.ctx.endpoints.getHttp('/discovery'), data.options),
            onApiUpdate: (rows, data) => this.ctx.httpClient.put(this.ctx.endpoints.getHttp(`/discovery/${data.service}`), data.options),
            onApiDelete: (rows, data) => this.ctx.httpClient.delete(this.ctx.endpoints.getHttp(`/discovery/${data.service}`)),
            'onClick:row': ({ row }) => this.ctx.routing.openURL(this.ctx.endpoints.getHttp(`/service/${row.service}`)),
          })
        ]),
      ])
      ]
    },
  }
})