// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    props: [
      'design',
      'serviceUuids',
      'item',
      'computedObject',
      'panel',
    ],

    icon: 'mdi-chart-timeline-variant-shimmer',
    description: 'diagramSequenceDescription',
    
    config() {
      return {
        startPanel: 'properties',
        panels: [
          {
            header: 'properties',
            name: 'properties',
            items: [
              {
                type: 'navigation',
                props: {
                  label: 'general',
                  caption: 'chart',
                  to: 'general',
                }
              },
              {
                type: 'navigation',
                props: {
                  label: 'xaxis',
                  caption: 'diagramSequenceXaxisSettings',
                  to: 'xaxis',
                }
              },
              {
                type: 'navigation',
                props: {
                  label: 'yaxis',
                  caption: 'diagramSequenceYaxisSettings',
                  to: 'yaxis',
                }
              },
              {
                type: 'navigation',
                props: {
                  label: 'dataSeries',
                  caption: 'diagramSequenceDataSeries',
                  to: 'dataSeries',
                }
              },
            ]
          },
          {
            header: 'general',
            name: 'general',
            items: [
              {
                type: 'form',
                props: {
                  scope: 'data',
                  schema: [
                    {
                      type: 'text',
                      label: 'width',
                      default: '',
                      hint: '100px, 100vw, 100%, ...',
                      scope: 'attrs.options.chart.width',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'text',
                      label: 'height',
                      default: '',
                      hint: '100px, 100vw, 100%, ...',
                      scope: 'attrs.options.chart.height',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'color',
                      label: 'backgroundColor',
                      default: '',
                      scope: 'attrs.options.chart.backgroundColor',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'color',
                      label: 'textColor',
                      default: '',
                      scope: 'attrs.options.chart.textColor',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'number',
                      label: 'borderRadius',
                      default: 0,
                      scope: 'attrs.options.chart.borderRadius',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                  ]
                }
              }
            ]
          },
          {
            header: 'xaxis',
            name: 'xaxis',
            items: [
              {
                type: 'form',
                props: {
                  scope: 'data',
                  schema: [
                    {
                      type: 'label',
                      label: 'title',
                      col: 'col-12',
                    },
                    {
                      type: 'text',
                      label: 'label',
                      default: '',
                      scope: 'attrs.options.xaxis.title.text',
                      sideSlot: 'link',
                      col: 'col-12',
                    },
                    {
                      type: 'text',
                      label: 'unit',
                      default: '',
                      scope: 'attrs.options.xaxis.title.unit',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'color',
                      label: 'color',
                      default: '',
                      scope: 'attrs.options.xaxis.title.style.color',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'label',
                      label: 'input',
                      col: 'col-12',
                    },
                    {
                      type: 'text',
                      label: 'label',
                      default: 'input',
                      scope: 'attrs.input.label',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'number',
                      label: 'value',
                      default: 0,
                      'call.step': 'return Number(`0.${"1".padStart(ctx.data.attrs.input.digits || 3, "0")}`)',
                      scope: 'attrs.input.value',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'number',
                      label: 'min',
                      default: undefined,
                      'call.step': 'return Number(`0.${"1".padStart(ctx.data.attrs.input.digits || 3, "0")}`)',
                      scope: 'attrs.input.min',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'number',
                      label: 'max',
                      default: undefined,
                      'call.step': 'return Number(`0.${"1".padStart(ctx.data.attrs.input.digits || 3, "0")}`)',
                      scope: 'attrs.input.max',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'number',
                      label: 'digits',
                      default: undefined,
                      scope: 'attrs.input.digits',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'text',
                      label: 'unit',
                      default: undefined,
                      scope: 'attrs.input.sideLabel',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'checkbox',
                      label: 'readonly',
                      default: true,
                      scope: 'attrs.input.readonly',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'color-palette',
                      default: '',
                      label: 'color',
                      scope: 'attrs.input.bgColor',
                      col: 'col-12',
                      sideSlot: 'link',
                    },
                    {
                      type: 'checkbox',
                      label: 'showShiftButtons',
                      default: true,
                      scope: 'attrs.input.shift',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'number',
                      label: 'shiftValue',
                      default: 1,
                      scope: 'attrs.input.shiftValue',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'label',
                      label: 'drag',
                      col: 'col-12',
                    },
                    {
                      type: 'checkbox',
                      label: 'enabled',
                      default: false,
                      scope: 'attrs.options.chart.draggable.x.enable',
                      sideSlot: 'link',
                      col: 'col-12',
                    },
                    {
                      type: 'number',
                      label: 'min',
                      default: undefined,
                      scope: 'attrs.options.chart.draggable.x.min',
                      sideSlot: 'link',
                      col: 'col-12 col-md-4',
                    },
                    {
                      type: 'number',
                      label: 'max',
                      default: undefined,
                      scope: 'attrs.options.chart.draggable.x.max',
                      sideSlot: 'link',
                      col: 'col-12 col-md-4',
                    },
                    {
                      type: 'number',
                      label: 'step',
                      default: 1,
                      scope: 'attrs.options.chart.draggable.y.steps',
                      sideSlot: 'link',
                      col: 'col-12 col-md-4',
                    },
                    {
                      type: 'label',
                      label: 'xaxis',
                      col: 'col-12',
                    },
                    {
                      type: 'number',
                      label: 'min',
                      default: 0,
                      scope: 'attrs.options.xaxis.min',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'number',
                      label: 'max',
                      default: 100,
                      scope: 'attrs.options.xaxis.max',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                  ]
                }
              }
            ]
          },
          {
            header: 'yaxis',
            name: 'yaxis',
            items: [
              {
                type: 'form',
                props: {
                  scope: 'data',
                  schema: [
                    {
                      type: 'label',
                      label: 'title',
                      col: 'col-12',
                    },
                    {
                      type: 'text',
                      label: 'label',
                      default: '',
                      scope: 'attrs.options.yaxis.title.text',
                      sideSlot: 'link',
                      col: 'col-12',
                    },
                    {
                      type: 'text',
                      label: 'unit',
                      default: '',
                      scope: 'attrs.options.yaxis.title.unit',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'color',
                      label: 'color',
                      default: '',
                      scope: 'attrs.options.yaxis.title.style.color',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'label',
                      label: 'output',
                      col: 'col-12',
                    },
                    {
                      type: 'text',
                      label: 'label',
                      default: 'output',
                      scope: 'attrs.output.label',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'number',
                      label: 'value',
                      default: 0,
                      'call.step': 'return Number(`0.${"1".padStart(ctx.data.attrs.output.digits || 3, "0")}`)',
                      scope: 'attrs.output.value',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'number',
                      label: 'min',
                      default: 0,
                      'call.step': 'return Number(`0.${"1".padStart(ctx.data.attrs.output.digits || 3, "0")}`)',
                      scope: 'attrs.output.min',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'number',
                      label: 'max',
                      default: 100,
                      'call.step': 'return Number(`0.${"1".padStart(ctx.data.attrs.output.digits || 3, "0")}`)',
                      scope: 'attrs.output.max',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'number',
                      label: 'digits',
                      default: 0,
                      scope: 'attrs.output.digits',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'text',
                      label: 'unit',
                      default: undefined,
                      scope: 'attrs.output.sideLabel',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'checkbox',
                      label: 'readonly',
                      default: true,
                      scope: 'attrs.output.readonly',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'color-palette',
                      default: '',
                      label: 'color',
                      scope: 'attrs.output.bgColor',
                      col: 'col-12',
                      sideSlot: 'link',
                    },
                    {
                      type: 'checkbox',
                      label: 'showShiftButtons',
                      default: true,
                      scope: 'attrs.output.shift',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'number',
                      label: 'shiftValue',
                      default: 5,
                      scope: 'attrs.output.shiftValue',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'label',
                      label: 'drag',
                      col: 'col-12',
                    },
                    {
                      type: 'checkbox',
                      label: 'enabled',
                      default: true,
                      scope: 'attrs.options.chart.draggable.y.enable',
                      sideSlot: 'link',
                      col: 'col-12 col-md-12',
                    },
                    {
                      type: 'number',
                      label: 'min',
                      default: 0,
                      scope: 'attrs.options.chart.draggable.y.min',
                      sideSlot: 'link',
                      col: 'col-12 col-md-4',
                    },
                    {
                      type: 'number',
                      label: 'max',
                      default: 100,
                      scope: 'attrs.options.chart.draggable.y.max',
                      sideSlot: 'link',
                      col: 'col-12 col-md-4',
                    },
                    {
                      type: 'number',
                      label: 'step',
                      default: 5,
                      scope: 'attrs.options.chart.draggable.y.steps',
                      sideSlot: 'link',
                      col: 'col-12 col-md-4',
                    },
                    {
                      type: 'label',
                      label: 'yaxis',
                      col: 'col-12',
                    },
                    {
                      type: 'number',
                      label: 'min',
                      default: -20,
                      scope: 'attrs.options.yaxis.min',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },
                    {
                      type: 'number',
                      label: 'max',
                      default: 110,
                      scope: 'attrs.options.yaxis.max',
                      sideSlot: 'link',
                      col: 'col-12 col-md-6',
                    },       
                  ]
                }
              }
            ]
          },
          {
            header: 'dataSeries',
            name: 'dataSeries',
            items: [
              {
                type: 'form',
                props: {
                  scope: 'data',
                  schema: [
                    {
                      type: 'collection',
                      scope: 'attrs.series',
                      label: 'dataSeries',
                      enableCopySelected: true,
                      'onClick:item': (ev = { index: 0}) => {
                        this.ctx.index['attrs.series'] = ev.index
                      },
                      'onRemove:item': (ev = { index: 0 }) => {
                        const prop = `series.${ev.index}`;
                        Object.keys(this.ctx.data.computed || {}).forEach((computedProp) => {
                          if (computedProp.startsWith(prop)) {
                            delete this.ctx.data.computed[computedProp]
                          }
                        })
                      },
                      'onRemove:items': (ev = { indexes: [] }) => {
                        ev.indexes.forEach((index) => {
                          const prop = `series.${index}`;
                          Object.keys(this.ctx.data.computed || {}).forEach((computedProp) => {
                            if (computedProp.startsWith(prop)) {
                              delete this.ctx.data.computed[computedProp]
                            }
                          })
                        })
                      },
                      default: [],
                      defaultValue: {
                        name: '',
                        data: [],
                        id: '0',
                        yaxis: '0',
                        color: '',
                        width: 2,
                        unit: '',
                        marker: {
                          enabled: true,
                          width: 20,
                          height: 20,
                          radius: 24,
                          symbol: 'circle',
                        },
                        dataLabels: {
                          enabled: true,
                          align: 'center',
                          verticalAlign: 'middle',
                          format: '{y:.1f}',
                          style: {
                            fontSize: '11px',
                            textOutline: 'none',
                            color: '#fff',
                          },
                        },
                      },
                      seamless: true,
                      nested: {
                        form: [
                          {
                            element: 'NgFormInput',
                            attrs: {
                              scope: 'attrs.series.{{attrs.props.selectedIndex}}.name',
                              col: 'col-12 col-md-6',
                              label: 'name',
                              type: 'text',
                              default: 'Serie {{attrs.props.selectedIndex}}'
                            },
                            sideSlot: 'link',
                          },
                          {
                            element: 'NgFormInput',
                            attrs: {
                              scope: 'attrs.series.{{attrs.props.selectedIndex}}.unit',
                              col: 'col-12 col-md-6',
                              label: 'unit',
                              type: 'text',
                            },
                            sideSlot: 'link',
                          },
                          {
                            element: 'NgFormInput',
                            attrs: {
                              scope: 'attrs.series.{{attrs.props.selectedIndex}}.dataLabels.format',
                              default: '{y:.1f}',
                              col: 'col-12 col-md-6',
                              label: 'format',
                              type: 'text',
                            },
                            sideSlot: 'link',
                          },
                          {
                            element: 'NgFormInput',
                            attrs: {
                              scope: 'attrs.series.{{attrs.props.selectedIndex}}.color',
                              col: 'col-12 col-md-6',
                              label: 'color',
                              type: 'color',
                              default: '#ff3333',
                            },
                            sideSlot: 'link',
                          },
                          {
                            element: 'NgFormInput',
                            attrs: {
                              scope: 'attrs.series.{{attrs.props.selectedIndex}}.marker.symbol',
                              default: 'circle',
                              col: 'col-12 col-md-6',
                              label: 'symbol',
                              type: 'select',
                              options: [
                                {
                                  label: 'circle',
                                  value: 'circle',
                                },
                                {
                                  label: 'triangle',
                                  value: 'triangle',
                                },
                                {
                                  label: 'square',
                                  value: 'square',
                                },
                              ]
                            },
                          },
                          {
                            element: 'NgFormInput',
                            attrs: {
                              scope: 'attrs.series.{{attrs.props.selectedIndex}}.marker.radius',
                              default: 24,
                              col: 'col-12 col-md-6',
                              label: 'radius',
                              type: 'number',
                            },
                          },
                          {
                            element: 'NgFormInput',
                            attrs: {
                              scope: 'attrs.series.{{attrs.props.selectedIndex}}.visible',
                              default: true,
                              col: 'col-12 col-md-6',
                              label: 'visible',
                              type: 'checkbox',
                            },
                            sideSlot: 'link',
                          },
                          {
                            element: 'NgFormInput',
                            attrs: {
                              type: 'collection',
                              label: 'datapoints',
                              scope: 'attrs.series.{{attrs.props.selectedIndex}}.data',
                              col: 'col-12',
                              default: [],
                              defaultValue: { x: 0, y: 0 },
                              enableCopySelected: true,
                              seamless: true,
                              'onRemove:item': (ev = { index: 0 }) => {
                                const prop = `series.${this.ctx.index['attrs.series']}.data.${ev.index}`;
                                Object.keys(this.ctx.data.computed || {}).forEach((computedProp) => {
                                  if (computedProp.startsWith(prop)) {
                                    delete this.ctx.data.computed[computedProp]
                                  }
                                })
                              },
                              'onRemove:items': (ev = { indexes: [] }) => {
                                ev.indexes.forEach((index) => {
                                  const prop = `series.${this.ctx.index['attrs.series']}.data.${index}`;
                                  Object.keys(this.ctx.data.computed || {}).forEach((computedProp) => {
                                    if (computedProp.startsWith(prop)) {
                                      delete this.ctx.data.computed[computedProp]
                                    }
                                  })
                                })
                              },
                            },
                            nested: {
                              form: [
                                {
                                  element: 'NgFormInput',
                                  attrs: {
                                    scope: 'attrs.series.{{attrs.props.selectedIndex}}.data.{{attrs.props.props.selectedIndex}}.x',
                                    col: 'col-12 col-md-6',
                                    label: 'X',
                                    hint: 'input',
                                    type: 'number',
                                    autofocus: true,
                                    default: 0,
                                  },
                                  sideSlot: 'link',
                                },
                                {
                                  element: 'NgFormInput',
                                  attrs: {
                                    scope: 'attrs.series.{{attrs.props.selectedIndex}}.data.{{attrs.props.props.selectedIndex}}.y',
                                    col: 'col-12 col-md-6',
                                    label: 'Y',
                                    hint: 'output',
                                    type: 'number',
                                    autofocus: true,
                                    default: 0,
                                  },
                                  sideSlot: 'link',
                                }
                              ],
                              item: [
                                {
                                  innerText: '{{attrs.props.props.displayIndex}}. {{$t:datapoint}}',
                                }
                              ],
                            }
                          },
                        ],
                        item: [
                          {
                            innerText: '{{attrs.props.displayIndex}}. {{attrs.props.item.name}}',
                          }
                        ],
                      }
                    }
                  ]
                }
              }
            ]
          }
        ]
      }
    },

    
    // Watch primitive or object values for it state changes...
    watch: {
      // props
      'item.attrs.series': {
        handler(val) {
          this.series = this.getSortedSeries(val)
          this.fixRangeX()
          this.fixRangeY()
          this.handleShiftButtons()
        },
        deep: true,
      },
      'item.attrs.options': {
        handler(val) {
          this.options = val
          this.options.xaxis.min = this.options.xaxis.min || 0;
          this.options.xaxis.max = this.options.xaxis.max || 100; 
          this.options.yaxis.min = this.options.yaxis.min || 0;
          this.options.yaxis.max =  this.options.yaxis.max || 100;
          this.fixRangeX()
          this.fixRangeY()
          this.handleShiftButtons()
        },
        deep: true,
      },
      'item.attrs.input': {
        handler() {
          this.setAnnotations()
        },
        deep: true,
      },
      'item.attrs.output': {
        handler() {
          this.setAnnotations()
        },
        deep: true,
      },
      shiftLoading: {
        handler(val) {
          if (val) {
            setTimeout(() => {
              this.shiftLoading = false;
            }, 2000)
          }
        },
      }
    },
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        shiftLoading: false,
        disableShift: {
          left: false,
          up: false,
          right: false,
          down: false,
        },
        disableShiftPerSeries: {},
        dialog: {
          shiftSerie: false,
        },
        shiftSerieIdx: 0,
        series: [],
        options: {
          chart: {
            type: 'line',
            events: {
              dataPointDrop() {
                // dummy event, we assign the method within mounted hook
              },
            },
            draggable: {
              y: {
                enable: true,
                min: 0,
                max: 100,
                steps: 1,
              },
              x: {
                enable: false,
                min: 0,
                max: 100,
                steps: 1,
              },
            },
            animation: false,
            zoom: {
              enabled: false,
            },
          },
          xaxis: {
            type: 'linear',
            title: {
              text: this.$t('input'),
              style: {
                  color: 'blue'
              }
            },
            labels: {
                style: {
                    colors: 'blue'
                }
            }
          },
          tooltip: {
            enabled: false,
          },
          yaxis: {
              min: 0,
              max: 100,
              id: '0',
              title: {
                text: this.$t('output'),
                rotate: -90,
                style: {
                  color: 'red'
                }
              },
              labels: {
                style: {
                    colors: 'red'
                }
              }
            },
        },
      }
    },
    
    // Define methods for the ui...
    methods: {
      getSortedSeries(series) {
        return series
      },
      getDragMinMax() {
        if (!(this.item.attrs.options && this.item.attrs.options.chart)) {
          return null
        }
        
        return {
          x: {
            min: this.item.attrs.options.chart.draggable.x.min || 0,
            max: this.item.attrs.options.chart.draggable.x.max || 100,
          },
          y: {
            min: this.item.attrs.options.chart.draggable.y.min || 0,
            max: this.item.attrs.options.chart.draggable.y.max || 100,
          }
        }
      },
      
      getSerie(attrs = {}) {
        return {
          ...(attrs || {}),
          id: this.ctx.uuidv4(),
          yaxis: '0',
          width: 2,
          marker: {
            ...(attrs.marker || {}),
            enabled: true,
            width: 20,
            height: 20,
          },
          dataLabels: {
            enabled: true,
            align: 'center',
            verticalAlign: 'middle',
            format: attrs.dataLabels.format,
            style: {
              fontSize: '11px',
              textOutline: 'none',
              color: '#fff',
            },
          },
        }
      },
      dataPointDrop(e = {series: {}, x: 0, y: 0, index: 0}) {
        if (!Array.isArray(this.item.attrs.series)) {
          return
        }
        
        const serieIndex = this.item.attrs.series.findIndex((serie) => e.series.id === serie.id)
        const setPayloadByProcessUuids = {}

        if (serieIndex >= 0) {
          const propX = `series.${serieIndex}.data.${e.index}.x`
          const propY = `series.${serieIndex}.data.${e.index}.y`
          
          // Correct value to avoid less or greater values according the siblings data items.
          const prevDataIndex = e.index - 1;
          const nextDataIndex = e.index + 1;
          let prevDataValue = { x: undefined, y: undefined };
          let nextDataValue = { x: undefined, y: undefined };
          let correctedX = e.x;

          if (prevDataIndex >= 0) {
            prevDataValue = this.series[serieIndex].data[prevDataIndex]
          }
  
          if (nextDataIndex < this.series[serieIndex].data.length) {
            nextDataValue = this.series[serieIndex].data[nextDataIndex]
          }

          if (prevDataValue.x === Number(prevDataValue.x) && e.x <= prevDataValue.x) {
            return
          }

          if (nextDataValue.x === Number(nextDataValue.x) && e.x >= nextDataValue.x) {
            return
          }
          
          if (this.item.attrs.options.chart.draggable.x.enable) {
            this.computedObject.getLinksByProperty(propX).forEach((link) => {
              if (!setPayloadByProcessUuids[link.processUuid]) {
                setPayloadByProcessUuids[link.processUuid] = {
                  link,
                  set: [],
                }
              }

              setPayloadByProcessUuids[link.processUuid].set.push({
                path: link.path,
                value: correctedX === null ? 0 : correctedX,
              })
            })
          }


          if (this.item.attrs.options.chart.draggable.y.enable) {
            this.computedObject.getLinksByProperty(propY).forEach((link) => {
              if (!setPayloadByProcessUuids[link.processUuid]) {
                setPayloadByProcessUuids[link.processUuid] = {
                  link,
                  set: [],
                }
              }

              setPayloadByProcessUuids[link.processUuid].set.push({
                path: link.path,
                value: e.y === null ? 0 : e.y,
              })
            })
          }
        }

        Object.values(setPayloadByProcessUuids).forEach((payload) => {
          this.computedObject.publish(payload.link, {
            set: payload.set,
          })
        })
      },

      fixRangeX() {
        const limits = this.getDragMinMax()

        if (!limits) {
          return
        }

        const minX = limits.x.min
        const maxX = limits.x.max
        
        const setPayloadByProcessUuids = {}

        this.item.attrs.series.forEach((serie, serieIndex) => {
          serie.data.forEach((data, dataIndex) => {
            if (data.x < minX && minX === Number(minX)) {
              data.x = minX
            } else if (data.x > maxX && maxX === Number(maxX)) {
              data.x = maxX
            }

            const prop = `series.${serieIndex}.data.${dataIndex}.x`
            this.computedObject.getChildsByProperty(prop).forEach((child) => {
              let val = child.getValue()
              let hasNewVal = false
              if (val < minX && minX === Number(minX)) {
                val = minX
                hasNewVal = true
              } else if (val > maxX && maxX === Number(maxX)) {
                val = maxX
                hasNewVal = true
              }

              if (hasNewVal) {
                const link = child.getLink()
                if (!setPayloadByProcessUuids[link.processUuid]) {
                  setPayloadByProcessUuids[link.processUuid] = {
                    link,
                    set: [],
                  }
                }

                setPayloadByProcessUuids[link.processUuid].set.push({
                  path: link.path,
                  value: val === null ? 0 : val
                })
              }
            })
          })
        })

        Object.values(setPayloadByProcessUuids).forEach((payload) => {
          this.computedObject.publish(payload.link, {
            set: payload.set,
          })
        })
      },

      fixRangeY() {
        if (!Array.isArray(this.item.attrs.series)) {
          return
        }

        if (!(this.item.attrs.options && this.item.attrs.options.yaxis)) {
          return
        }

        const limits = this.getDragMinMax()

        if (!limits) {
          return
        }
        
        const minY = limits.y.min
        const maxY = limits.y.max

        const setPayloadByProcessUuids = {}

        this.item.attrs.series.forEach((serie, serieIndex) => {
          serie.data.forEach((data, dataIndex) => {
            const prop = `series.${serieIndex}.data.${dataIndex}.y`

            if (data.y < minY && minY === Number(minY)) {
              data.y = minY
            } else if (data.y > maxY && maxY === Number(maxY)) {
              data.y = maxY
            }
            
            this.computedObject.getChildsByProperty(prop).forEach((child) => {
              let val = child.getValue()
              let hasNewVal = false
              if (val < minY && minY === Number(minY)) {
                val = minY
                hasNewVal = true
              } else if (val > maxY && maxY === Number(maxY)) {
                val = maxY
                hasNewVal = true
              }

              if (hasNewVal) {
                const link = child.getLink()
                if (!setPayloadByProcessUuids[link.processUuid]) {
                  setPayloadByProcessUuids[link.processUuid] = {
                    link,
                    set: [],
                  }
                }

                setPayloadByProcessUuids[link.processUuid].set.push({
                  path: link.path,
                  value: val === null ? 0 : val
                })
              }
            })
          })
        })

        Object.values(setPayloadByProcessUuids).forEach((payload) => {
          this.computedObject.publish(payload.link, {
            set: payload.set,
          })
        })
      },

      handleShiftButtons() {
        const limits = this.getDragMinMax()

        if (!limits) {
          return
        }

        const xMinItems = []
        const xMaxItems = []
        const yMinItems = []
        const yMaxItems = []
        
        this.series.forEach((serie, idx) => {
          if (serie.data && serie.data.length >= 2) {
            const minXItem = serie.data.reduce((a,b)=>a.x<b.x?a:b)
            const maxXItem = serie.data.reduce((a,b)=>a.x>b.x?a:b)
            const minYItem = serie.data.reduce((a,b)=>a.y<b.y?a:b)
            const maxYItem = serie.data.reduce((a,b)=>a.y>b.y?a:b)
            
            if (!this.disableShiftPerSeries[idx]) {
              this.disableShiftPerSeries[idx] = {
                up: false,
                down: false,
                left: false,
                right: false,
              }
            }

            this.disableShiftPerSeries[idx].left = !(minXItem.x > limits.x.min)
            this.disableShiftPerSeries[idx].right = !(maxXItem.x < limits.x.max)
            this.disableShiftPerSeries[idx].down = !(minYItem.y > limits.y.min)
            this.disableShiftPerSeries[idx].up = !(maxYItem.y < limits.y.max)

            xMinItems.push(minXItem)
            xMaxItems.push(maxXItem)
            yMinItems.push(minYItem)
            yMaxItems.push(maxYItem)
          }
        })

        this.disableShift.left = !xMinItems.find((item) => item.x > limits.x.min)
        this.disableShift.right = !xMaxItems.find((item) => item.x < limits.x.max)
        this.disableShift.down = !yMinItems.find((item) => item.y > limits.y.min)
        this.disableShift.up = !yMaxItems.find((item) => item.y < limits.y.max)
        this.shiftLoading = false;
      },

      setAnnotations() {
        if (!(this.item.attrs.input && this.item.attrs.options && this.item.attrs.options.xaxis)) {
          return
        }

        this.item.attrs.options.annotations = {
          xaxis: [
            {
              x: this.item.attrs.input.value,
              label: {
                text: `${this.ctx.i18n.t(this.item.attrs.options.xaxis.title.text)}: ${this.item.attrs.input.value}`,
              },
              fillColor: this.item.attrs.options.xaxis.title.style.color || 'black',
              dashed: true,
            },
          ],
          yaxis: [
            {
              y: this.item.attrs.output.value,
              label: {
                text: `${this.ctx.i18n.t(this.item.attrs.options.yaxis.title.text)}: ${this.item.attrs.output.value}`,
              },
              fillColor: this.item.attrs.options.yaxis.title.style.color || 'black',
              dashed: true,
            },
          ],
        }
      },

      onShiftSeries(axis = 'x', value, series) {
        const localSeries = series || this.series
        if (!Array.isArray(localSeries)) {
          return
        }

        const limits = this.getDragMinMax()

        if (!limits) {
          return
        }

        let min = limits[axis].min
        let max = limits[axis].max
        
        const clonedSeries = JSON.parse(JSON.stringify(localSeries))
        const setPayloadByProcessUuids = {}
        this.shiftLoading = true
        clonedSeries.forEach((serie) => {
          const serieIndex = this.item.attrs.series.findIndex((s) => serie.id === s.id)
          if (serie.data && serie.data.length >= 2) {
            const minItem = serie.data.reduce((a,b)=>a[axis]<b[axis]?a:b)
            const maxItem = serie.data.reduce((a,b)=>a[axis]>b[axis]?a:b)
            const diffMin = min - minItem[axis]
            const diffMax = max - maxItem[axis]
            if (value > 0 && value > diffMax) {
              value = diffMax
            } else if (value < 0 && value < diffMin) {
              value = diffMin
            }
  
            serie.data.forEach((_data, dataIndex) => {
              const prop = `series.${serieIndex}.data.${dataIndex}.${axis}`
              let newValue = serie.data[dataIndex][axis] + value
  
              if (newValue < min) {
                newValue = min
              } else if (newValue > max) {
                newValue = max
              }
              
              this.computedObject.getChildsByProperty(prop).forEach((child) => {
                  const link = child.getLink()
                  if (!setPayloadByProcessUuids[link.processUuid]) {
                    setPayloadByProcessUuids[link.processUuid] = {
                      link,
                      set: [],
                    }
                  }

                  setPayloadByProcessUuids[link.processUuid].set.push({
                    path: link.path,
                    value: newValue === null ? 0 : newValue
                  })
              })
            })           
          }
        })
        Object.values(setPayloadByProcessUuids).forEach((payload) => {
          this.computedObject.publish(payload.link, {
            set: payload.set,
          })
        })
      },

      onShift(direction, serieIdx, val) {
        let xIncr = val || this.item.attrs.input.shiftValue || 1
        let yIncr = val || this.item.attrs.output.shiftValue || 1
        const localSeries = serieIdx !== undefined ? [this.series[serieIdx]] : undefined;
        switch (direction) {
          case 'left':
            xIncr *= -1
            this.onShiftSeries('x', xIncr, localSeries)
            break;
          case 'right':
            this.onShiftSeries('x', xIncr, localSeries)
            break;
          case 'up':
            this.onShiftSeries('y', yIncr, localSeries)
            break;
          case 'down':
            yIncr *= -1
            this.onShiftSeries('y', yIncr, localSeries)
            break;
        }
      },

      onApply(serieIdx) {
        const localSerie = this.series[serieIdx];
        const applySeries = this.series.filter((serie) => serie.id !== localSerie.id)

        const setPayloadByProcessUuids = {}
        this.shiftLoading = true
        applySeries.forEach((serie) => {
          const serieIndex = this.item.attrs.series.findIndex((s) => serie.id === s.id)


          localSerie.data.forEach((data, dataIndex) => {
            if (dataIndex < serie.data.length) {
              const propX = `series.${serieIndex}.data.${dataIndex}.x`
              const propY = `series.${serieIndex}.data.${dataIndex}.y`
              const x = data.x
              const y = data.y
              
              this.computedObject.getChildsByProperty(propX).forEach((child) => {
                  const link = child.getLink()
                  if (!setPayloadByProcessUuids[link.processUuid]) {
                    setPayloadByProcessUuids[link.processUuid] = {
                      link,
                      set: [],
                    }
                  }
  
                  setPayloadByProcessUuids[link.processUuid].set.push({
                    path: link.path,
                    value: x === null ? 0 : x
                  })
              })

              this.computedObject.getChildsByProperty(propY).forEach((child) => {
                const link = child.getLink()
                if (!setPayloadByProcessUuids[link.processUuid]) {
                  setPayloadByProcessUuids[link.processUuid] = {
                    link,
                    set: [],
                  }
                }

                setPayloadByProcessUuids[link.processUuid].set.push({
                  path: link.path,
                  value: y === null ? 0 : y
                })
              })
            }
          })           
        })
        Object.values(setPayloadByProcessUuids).forEach((payload) => {
          this.computedObject.publish(payload.link, {
            set: payload.set,
          })
        })
      }
    },

    mounted() {
      this.item.valueType = 'number'

      if (!this.item.attrs.series) {
        this.item.attrs.series = [];
      }

      // map attrs to chart
      this.item.attrs.options = this.ctx.merge(this.options, this.item.attrs.options || {})
      this.item.attrs.options.chart.events.dataPointDrop = this.dataPointDrop.bind(this)
      this.item.attrs.options.xaxis.labels.style.colors = this.item.attrs.options.xaxis.title.style.color || '#000'
      this.item.attrs.series = this.getSortedSeries(this.item.attrs.series.map((s) => this.getSerie(s)))
      this.options = this.item.attrs.options
    },
  
	
      // Listen on mounted event...
    unmounted() {
      // logic
    },
    
    // Define elements within the ui configuration...
    nodes() {
      this.item.attrs.input = this.item.attrs.input || {}
      this.item.attrs.output = this.item.attrs.output || {}
      
      return [
        // any ui component node can be used...
        {
          type: 'div',
          props: {
            class: 'row q-col-gutter-sm',
          },
          children: {
            default: () => {
              return [
                {
                  type: 'div',
                  props: {
                    class: `col-12 ${this.item.attrs.input.shift || this.item.attrs.output.shift ? 'col-md-11' : ''}`,
                  },
                  children: {
                    default: () => [
                      {
                        type: 'NgChart',
                        props: {
                          opts: this.options,
                          series: this.series,
                          disableMergeOptions: true,
                          disableToolbar: true,
                          class: 'full-width',
                          configKey: '',
                          onResize: (newOptions) => {
                            this.options.chart.width = newOptions.width
                          }
                        },
                      }
                    ]
                  }
                },
                this.item.attrs.input.shift || this.item.attrs.output.shift ? {
                  type: 'div',
                  props: {
                    class: 'col-12 col-md-1 flex justify-center',
                  },
                  children: {
                    default: () => {
                      return [
                        // child nodes or string content
                        {
                          type: 'div',
                          props: {
                            class: 'column q-gutter-md',
                            flat: true,
                          },
                          children: {
                            default: () => {
                              return [
                                this.item.attrs.output.shift ? {
                                  type: 'NgActionButton',
                                  props: {
                                    icon: 'mdi-chevron-up',
                                    onClick: () => this.onShift('up'),
                                    disable: this.disableShift.up,
                                    loading: this.shiftLoading,
                                  },
                                } : '',
                                this.item.attrs.output.shift ? {
                                  type: 'NgActionButton',
                                  props: {
                                    icon: 'mdi-chevron-down',
                                    onClick: () => this.onShift('down'),
                                    disable: this.disableShift.down,
                                    loading: this.shiftLoading,
                                  },
                                } : '',
                                this.item.attrs.input.shift ? {
                                  type: 'NgActionButton',
                                  props: {
                                    icon: 'mdi-chevron-left',
                                    onClick: () => this.onShift('left'),
                                    disable: this.disableShift.left,
                                    loading: this.shiftLoading,
                                  },
                                } : '',
                                this.item.attrs.input.shift ? {
                                  type: 'NgActionButton',
                                  props: {
                                    icon: 'mdi-chevron-right',
                                    onClick: () => this.onShift('right'),
                                    disable: this.disableShift.right,
                                    loading: this.shiftLoading,
                                  },
                                } : '',
                                this.item.attrs.input.shift || this.item.attrs.output.shift ? {
                                  type: 'NgMenu',
                                  props: {
                                    icon: 'more_vert',
                                    persistent: true,
                                    color: 'primary',
                                    tooltip: 'dataSeries',
                                    items: [
                                      ...this.item.attrs.series.map((serie, serieIdx) => ({
                                        label: serie.name,
                                        id: serie.id,
                                        click: () => {
                                          this.shiftSerieIdx = serieIdx
                                          this.dialog.shiftSerie = true
                                        }
                                      }))
                                    ],
                                  },
                                } : '',
                              ]
                            }
                          }
                        },
                      ]
                    }
                  },
                } : '',
                {
                  type: 'NgFormInput',
                  props: {
                    ...(this.item.attrs.input || {}),
                    type: 'number',
                    class: 'col-12 col-md-6',
                    modelValue: this.item.attrs.input.value,
                    'onUpdate:modelValue': (val) =>  this.item.attrs.input.value = val,
                    'onBlur': () =>  {
                      this.computedObject.getLinksByProperty('input.value').forEach((link) => {
                        this.computedObject.publish(link, {
                          set: [
                            {
                              path: link.path,
                              value: this.item.attrs.input.value,
                            }
                          ]
                        })
                      })
                    },
                  },
                },
                {
                  type: 'NgFormInput',
                  props: {
                    ...(this.item.attrs.output || {}),
                    type: 'number',
                    class: 'col-12 col-md-6',
                    modelValue: this.item.attrs.output.value,
                    'onUpdate:modelValue': (val) =>  this.item.attrs.output.value = val,
                    'onBlur': () =>  {
                      this.computedObject.getLinksByProperty('output.value').forEach((link) => {
                        this.computedObject.publish(link, {
                          set: [
                            {
                              path: link.path,
                              value: this.item.attrs.output.value,
                            }
                          ]
                        })
                      })
                    },
                  },
                },
                {
                  type: 'NgDialog',
                  props: {
                    modelValue: this.dialog.shiftSerie,
                    'onUpdate:modelValue': (val) => this.dialog.shiftSerie = val,
                    draggable: true,
                    small: true,
                    noCancel: true,
                    seamless: false,
                    title: this.series[this.shiftSerieIdx] ? this.series[this.shiftSerieIdx].name : ''
                  },
                  children: {
                    default: () => [
                      {
                        type: 'div',
                        props: {
                          class: 'row q-col-gutter-md',
                        },
                        children: {
                          default: () => [
                            {
                              type: 'div',
                              props: {
                                class: 'col-12 q-gutter-md',
                              },
                              children: {
                                default: () => [
                                  {
                                    type: 'NgBadge',
                                    props: {
                                      style: {
                                        backgroundColor: `${this.series[this.shiftSerieIdx].color}!important`,
                                      },
                                      text: `${this.ctx.i18n.t('move')}: ${this.ctx.i18n.t(this.series[this.shiftSerieIdx].name)}`,
                                    }
                                  },
                                ]
                              }
                            },
                            {
                              type: 'div',
                              props: {
                                class: 'col-6 q-gutter-md column',
                              },
                              children: {
                                default: () => [
                                  this.item.attrs.output.shift ? {
                                    type: 'NgActionButton',
                                    props: {
                                      icon: 'mdi-chevron-up',
                                      onClick: () => this.onShift('up', this.shiftSerieIdx),
                                      disable: this.disableShiftPerSeries[this.shiftSerieIdx].up,
                                      loading: this.shiftLoading,
                                    },
                                  } : '',
                                  this.item.attrs.output.shift ? {
                                    type: 'NgActionButton',
                                    props: {
                                      icon: 'mdi-chevron-down',
                                      onClick: () => this.onShift('down', this.shiftSerieIdx),
                                      disable: this.disableShiftPerSeries[this.shiftSerieIdx].down,
                                      loading: this.shiftLoading,
                                    },
                                  } : '',
                                  this.item.attrs.input.shift ? {
                                    type: 'NgActionButton',
                                    props: {
                                      icon: 'mdi-chevron-left',
                                      onClick: () => this.onShift('left', this.shiftSerieIdx),
                                      disable: this.disableShiftPerSeries[this.shiftSerieIdx].left,
                                      loading: this.shiftLoading,
                                    },
                                  } : '',
                                  this.item.attrs.input.shift ? {
                                    type: 'NgActionButton',
                                    props: {
                                      icon: 'mdi-chevron-right',
                                      onClick: () => this.onShift('right', this.shiftSerieIdx),
                                      disable: this.disableShiftPerSeries[this.shiftSerieIdx].right,
                                      loading: this.shiftLoading,
                                    },
                                  } : '',
                                ]
                              }
                            },
                            {
                              type: 'div',
                              props: {
                                class: 'col-6 q-gutter-md column',
                              },
                              children: {
                                default: () => [
                                  this.item.attrs.output.shift ? {
                                    type: 'NgActionButton',
                                    props: {
                                      icon: 'mdi-chevron-double-up',
                                      onClick: () => this.onShift('up', this.shiftSerieIdx, this.getDragMinMax().y.max - this.getDragMinMax().y.min),
                                      disable: this.disableShiftPerSeries[this.shiftSerieIdx].up,
                                      loading: this.shiftLoading,
                                    },
                                  } : '',
                                  this.item.attrs.output.shift ? {
                                    type: 'NgActionButton',
                                    props: {
                                      icon: 'mdi-chevron-double-down',
                                      onClick: () => this.onShift('down', this.shiftSerieIdx, this.getDragMinMax().y.max - this.getDragMinMax().y.min),
                                      disable: this.disableShiftPerSeries[this.shiftSerieIdx].down,
                                      loading: this.shiftLoading,
                                    },
                                  } : '',
                                  this.item.attrs.input.shift ? {
                                    type: 'NgActionButton',
                                    props: {
                                      icon: 'mdi-chevron-double-left',
                                      onClick: () => this.onShift('left', this.shiftSerieIdx, this.getDragMinMax().x.max - this.getDragMinMax().x.min),
                                      disable: this.disableShiftPerSeries[this.shiftSerieIdx].left,
                                      loading: this.shiftLoading,
                                    },
                                  } : '',
                                  this.item.attrs.input.shift ? {
                                    type: 'NgActionButton',
                                    props: {
                                      icon: 'mdi-chevron-double-right',
                                      onClick: () => this.onShift('right', this.shiftSerieIdx, this.getDragMinMax().x.max - this.getDragMinMax().x.min),
                                      disable: this.disableShiftPerSeries[this.shiftSerieIdx].right,
                                      loading: this.shiftLoading,
                                    },
                                  } : '',
                                ]
                              }
                            },
                            {
                              type: 'div',
                              props: {
                                class: 'col-6 q-gutter-md column',
                              },
                              children: {
                                default: () => [
                                  {
                                    type: 'NgActionButton',
                                    props: {
                                      icon: 'mdi-arrow-expand-right',
                                      label: 'applyForAll',
                                      onClick: () => this.onApply(this.shiftSerieIdx),
                                      loading: this.shiftLoading,
                                    },
                                  },
                                ]
                              }
                            }
                          ]
                        }
                      },
                    ]
                  }
                }
              ]
            }
          },
        },
      ]
    },
  }
})