// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    props: [
      'design',
      'serviceUuids',
      'item',
      'computedObject',
      'panel',
    ],

    icon: 'mdi-label',

    config: {
      startPanel: 'properties',
      panels: [
        {
          header: 'properties',
          name: 'properties',
          items: [
            {
              type: 'form',
              props: {
                scope: 'data',
                schema: [
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'hidden',
                    scope: 'attrs.hidden',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'label',
                    scope: 'attrs.label',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'suffix',
                    scope: 'link.path',
                    hint: 'staticOrTemplatePathDescription',
                    col: 'col-12',
                  },
                  {
                    type: 'select',
                    default: '',
                    label: 'type',
                    scope: 'valueType',
                    col: 'col-12',
                    options: [
                      { label: '-', value: '' },
                      { label: 'number', value: 'number'},
                      { label: 'boolean', value: 'boolean'},
                      { label: 'string', value: 'string'},
                    ]
                  },
                  {
                    type: 'number',
                    default: '',
                    label: 'digits',
                    scope: 'attrs.digits',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'string',
                    default: '',
                    label: 'fontSize',
                    scope: 'attrs.fontSize',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'select',
                    default: '',
                    label: 'type',
                    scope: 'attrs.fontType',
                    col: 'col-12',
                    options: [
                      { label: '-', value: '' },
                      { label: 'bold', value: 'bold'},
                      { label: 'caption', value: 'caption'},
                      { label: 'overline', value: 'overline'},
                    ]
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'readPermissions',
                    hint: 'readPermissionsDescription',
                    scope: 'rights.read',
                    col: 'col-12',
                  },
                ]
              },
            }
          ]
        }
      ]
    },
	
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
	data() {
      return {
        // VAR
        value: this.item.attrs.label,
        digits: undefined,
      }
    },

    watch: {
      'item.attrs': {
        handler() {
          this.panel.setPermissions(this.item.rights, this.item)
        },
        deep: true,
      },
    },

    // Listen on mounted event...
    mounted() {
      // logic
      this.panel.on(this.computedObject, '', this.onEvent)
      this.digits = this.computedObject.getDigits()
      this.panel.setPermissions(this.item.rights, this.item)
    },
	
	// Listen on mounted event...
    unmounted() {
      this.panel.off(this.computedObject, '', this.onEvent)
    },
	
    methods: {
        onEvent(ev) {
          const value = this.panel.getEventValue(ev)
          if (typeof value === 'number') {
            this.value = value.toFixed(this.digits);
          } else {
            this.value = `${value || '...'}`;
          }
        }
    },
	
    // Define elements within the ui configuration...
    nodes() {
      return [
        // any ui component node can be used...
        {
          type: 'NgText',
          props: {
            ...this.item.attrs,
            text: this.item.attrs.label || this.value,
            style: {
              fontSize: this.item.attrs.fontSize,
            },
            type: this.item.attrs.fontType || '',
          },
        },
      ]
    },
  }
})