// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    props: [
      'design',
	  'serviceUuids',
      'item',
      'computedObject',
      'panel',
    ],

    icon: 'mdi-table',
    
    config: {
      startPanel: 'properties',
      panels: [
        {
          header: 'properties',
          name: 'properties',
          items: [
            {
              type: 'form',
              props: {
                scope: 'data',
                schema: [
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'hidden',
                    scope: 'attrs.hidden',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'disable',
                    scope: 'attrs.disable',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'suffix',
                    scope: 'link.path',
                    hint: 'staticOrTemplatePathDescription',
                    col: 'col-12',
                  },
                  {
                    type: 'select',
                    default: '70vh',
                    label: 'height',
                    scope: 'attrs.maxHeight',
                    col: 'col-12',
                    options: [
                      { label: 'small', value: '25vh' },
                      { label: 'medium', value: '50vh' },
                      { label: 'large', value: '70vh' },
                    ]
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'readPermissions',
                    hint: 'readPermissionsDescription',
                    scope: 'rights.read',
                    col: 'col-12',
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'writePermissions',
                    hint: 'writePermissionsDescription',
                    scope: 'rights.write',
                    col: 'col-12',
                  },
                ]
              },
            }
          ]
        }
      ]
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
        // VAR
        changelog: [],
        url: '',
      }
    },

    watch: {
      'item.attrs': {
        handler() {
          this.panel.setPermissions(this.item.rights, this.item)
        },
        deep: true,
      },
    },
    // Listen on mounted event...
    mounted() {
      // logic
      this.url = this.panel.getEndpoint(this.computedObject.getProcessUuid())
      this.computedObject.getPathes().forEach((path) => {
        this.changelog.push({
          group: '',
          filter: {
            path,
          }
        })
      })
      this.panel.setPermissions(this.item.rights, this.item)
    },
	
      // Listen on mounted event...
    unmounted() {
      // logic
    },
	
    methods: {
    },
	
    // Define elements within the ui configuration...
    nodes() {
      if (!this.url) {
        return [
          {
            type: 'NgSpinner',
            props: {
              label: 'loading',
            },
          }
        ]
      }
      return [
        // any ui component node can be used...
        {
          type: 'NgDMSTableChangelog',
          props: {
            ...this.item.attrs,
            workspaceUuid: this.design.ctx.workspaceUuid,
			serviceUuids: this.design.ctx.serviceUuids,
            changelog: this.changelog,
            url: this.url,
            allGroups: true,
            maxHeight: this.item.attrs.maxHeight,
          },
        },
      ]
    },
  }
})