// Call ngui function to create a ui config.
window.ngui(() => {
  return {
    props: [
      'design',
      'serviceUuids',
      'item',
      'computedObject',
      'panel',
      'parent',
    ],

    size: 12,

    icon: 'mdi-chart-line',
    
    config: {
      startPanel: 'properties',
      panels: [
        {
          header: 'properties',
          name: 'properties',
          onMounted: `
            if (!Array.isArray(ctx.data.serieNames)) {
              ctx.data.serieNames = []
            }
            return
          `,
          items: [
            {
              type: 'list',
              props: {
                scope: 'data.links',
                label: '{{ path }}',
                deletable: true,
                addable: {
                  to: 'link',
                  label: 'addLink',
                  sortable: true,
                  default: {
                    processManagerUuid: '',
                    instanceUuid: '',
                    processUuid: '',
                    path: '',
                  }
                },
                to: 'link',
                index: 'link'
              },
            },
            {
              type: 'form',
              props: {
                scope: 'data',
                schema: [
                  {
                    type: 'select',
                    label: 'type',
                    default: 'line',
                    scope: 'attrs.type',
                    col: 'col-12',
                    options: [
                      { label: 'lineChart', value: 'line' },
                      { label: 'columnChart', value: 'column' },
                      { label: 'pieChart', value: 'pie' },
                      { label: 'barChart', value: 'bar' },
                    ]
                  },
                  {
                    type: 'checkbox',
                    default: false,
                    label: 'hidden',
                    scope: 'attrs.hidden',
                    col: 'col-12',
                    sideSlot: 'link',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'configuration',
                    scope: 'attrs.configKey',
                    col: 'col-12',
                    hint: 'chartTemplateConfigKeyDescription',
                    sideSlot: 'link',
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'readPermissions',
                    hint: 'readPermissionsDescription',
                    scope: 'rights.read',
                    col: 'col-12',
                  },
                  {
                    type: 'permissions',
                    default: [],
                    label: 'writePermissions',
                    hint: 'writePermissionsDescription',
                    scope: 'rights.write',
                    col: 'col-12',
                  },
                ]
              },
            }
          ]
        },
        {
          header: 'link',
          name: 'link',
          items: [
            {
              type: 'form',
              props: {
                scope: 'data',
                showReset: false,
                schema: [
                  {
                    type: 'text',
                    default: '',
                    label: 'suffix',
                    scope: 'links.{{index.link}}.path',
                    col: 'col-12 col-md-6',
                  },
                  {
                    type: 'text',
                    default: '',
                    label: 'name',
                    scope: 'serieNames.{{index.link}}',
                    col: 'col-12 col-md-6',
                  },
                ]
              },
            }
          ]
        }
      ]
    },
    
    // The returned object can be accessed with this within any method...
    // Do not use this.ctx within this function because it is not yet created.
    data() {
      return {
		digits: undefined,
        chartNodes: [],
        url: '',
        localConfigKey: '',
      }
    },
    watch: {
      'item.attrs': {
        handler() {
          this.panel.setPermissions(this.item.rights, this.item)
        },
        deep: true,
      },
    },
    // Listen on mounted event...
    created() {
      // logic
      this.item.valueType = 'number'

      this.url = this.panel.getEndpoint(this.computedObject.getProcessUuid())
      this.digits = this.computedObject.getDigits()
      let nodesByEndpoint = {}
      const links = this.item.links || []
      const serieNames = this.item.serieNames || []
      
      // Create the dms-chart payload. We can define multiple chart series.
      links.forEach((link) => {
        if (this.url) {
          if (!nodesByEndpoint[this.url]) {
            nodesByEndpoint[this.url] = {
              url: this.url,
              processManagerUuid: link.processManagerUuid || this.computedObject.getProcessManagerUuid(),
              pathes: [],
              serieNames,
            }
          }
          nodesByEndpoint[this.url].pathes.push(this.computedObject.getPath(link.path))
        }

      })
      this.chartNodes = Object.values(nodesByEndpoint)
    },

    mounted() {
      this.panel.setPermissions(this.item.rights, this.item)
    },
	
      // Listen on mounted event...
    unmounted() {
      // logic
    },
	
    methods: {
      getConfigKey() {
        let location = '';
        if (!this.item.attrs.configKey) {
          return ''
        }
        if (this.parent.ctx.template && this.parent.ctx.template.template && this.parent.ctx.template.template.private) {
          location = `workspaces/${this.parent.ctx.workspaceUuid}/templates/charts`
        } else {
          location = `templates/charts`
        }

        return `${location}/${this.item.attrs.configKey}.json`
      }
    },
	
    // Define elements within the ui configuration...
    nodes() {
      if (!this.url) {
        return [
          {
            type: 'NgSpinner',
            props: {
              label: 'loading',
            },
          }
        ]
      }
      return [
        // any ui component node can be used...
        {
          type: 'NgDMSNodeChart',
          props: {
            ...this.item.attrs,
            workspaceUuid: this.design.ctx.workspaceUuid,
			serviceUuids: this.design.ctx.serviceUuids,
            configKey: this.getConfigKey(),
            digits: this.digits,
            nodes: this.chartNodes,
          },
        },
      ]
    },
  }
})