// Copyright 2022 MST Systemtechnik AG. All Rights Reserved.

window.addEventListener('load', function () {

    'use strict';

    let currentTabId = 0;
    let maxLines = 100;
    let showEmptyLines = false;
    document.getElementById('max-line-select').onchange = function(event) {
        maxLines = parseInt(event.target.value);
        // Remove log content
        document.getElementById('log-table-rows').innerHTML = "";
        // Request log for selected tab
        socket.send(JSON.stringify({ command: 'subscribeLog', logId: currentTabId,  maxLines: (showEmptyLines ? maxLines : maxLines * 2) }));
    };    

    document.getElementById('emtpy-lines-enabled').onchange = function(event) {
        showEmptyLines = document.getElementById('emtpy-lines-enabled').checked;

        // Remove log content
        document.getElementById('log-table-rows').innerHTML = "";
        // Request log for selected tab
        socket.send(JSON.stringify({ command: 'subscribeLog', logId: currentTabId, maxLines: (showEmptyLines ? maxLines : maxLines * 2) }));
    };    

    function removeSelection() {
        if (window.getSelection) {
            if (window.getSelection().empty) { // Chrome
                window.getSelection().empty();
            } else if (window.getSelection().removeAllRanges) { // Firefox
                window.getSelection().removeAllRanges();
            }
        } else if (document.selection) { // IE?
            document.selection.empty();
        }
    }

    document.getElementById('log-container').ondblclick = function(event) {
        let el = event.target;
        let trEl = undefined;
        if (el != undefined) {
            if (el.nodeName == 'TD') {
                trEl = el.parentElement;
            }
            else if (el.parentElement.nodeName == 'TD') {
                trEl = el.parentElement.parentElement;
            }
            if (trEl != undefined) {
                trEl.classList.toggle('marked');
                // Copy text
                removeSelection();
                if (document.selection) {
                    let range = document.body.createTextRange();
                    range.moveToElementText(trEl);
                    range.select().createTextRange();
                    document.execCommand('copy');
                } else if (window.getSelection) {
                    let range = document.createRange();
                    range.selectNode(trEl);
                    window.getSelection().addRange(range);
                    document.execCommand('copy');
                }       
                removeSelection();
            }
        }
    };

    // Error panel
    let modalTimer = 0;
    function showError(title, text, displayDuration) {
        let infoPanelEl = document.getElementById('error-panel');
        infoPanelEl.innerHTML = '<b>' + title + '</b><br>';
        infoPanelEl.innerHTML += text;
        infoPanelEl.className = 'sticky';
        // Start timer for auto close
        if (modalTimer) {
            clearTimeout(modalTimer);
        }
        if (displayDuration) {
            modalTimer = setTimeout(function () {
                infoPanelEl.className = 'sticky invisible';
            }, displayDuration);
        }
    }

    let urlParams = new URLSearchParams(window.location.search);
    let processUuid = urlParams.get('processUuid');
	let wsProtocol = 'ws:';
	if (location.protocol === 'https:') {
		wsProtocol = 'wss:';
	}
	let socket = new WebSocket(wsProtocol + '//' + location.host + location.pathname.substring(0, location.pathname.lastIndexOf('/')) + '/v1/logs/' + processUuid);

    window.addEventListener("pagehide", function () {
        socket.onclose = function () {};
        if (socket.readyState == WebSocket.OPEN)
        {
            socket.close();
        }
    });
    
    let firstLogs = true;
    socket.onopen = function () {
        // Init logs
        firstLogs = true;
        socket.send(JSON.stringify({ command: 'enumLogs' }));
    };

    const tabClick = (event) => {
        firstLogs = true;
        const tabButtonClicked = event.target;
        // First remove selected on tabs
        for (const elem of document.getElementsByClassName('tab')) {
            elem.classList.remove('selected');
        }
        // and remove log content
        document.getElementById('log-table-rows').innerHTML = "";
        // Now we set selected and show the selected tab.
        tabButtonClicked.classList.add('selected');
        // Request log for selected tab
        currentTabId = parseInt(tabButtonClicked.dataset.id);
        socket.send(JSON.stringify({ command: 'subscribeLog', logId: currentTabId,  maxLines: (showEmptyLines ? maxLines : maxLines * 2) }));
    };

    function createSpanElementWithText(text) {
        let element = document.createElement('span');
        element.innerHTML = text;
        return element;
    }

    function htmlEncode(str) {
        return str.replace(/&/g, "&amp;").replace(/>/g, "&gt;").replace(/</g, "&lt;").replace(/"/g, "&quot;");
    }

    socket.onmessage = function (event) {
        let response = JSON.parse(event.data);

        if (response.event == 'enumLogs') {
            // Create new tabs
            let logEl = document.getElementById('log-container');
            if (response.logs.length == 0) {
                logEl.innerHTML = 'No Log data for this process!';
            }
            else {
                logEl.innerHTML = '';
                // Create buttons
                let i = 1;
                response.logs.forEach(function (log) {
                    let tabButtonEl = document.createElement('div');
                    tabButtonEl.id = 'tab' + i;
                    tabButtonEl.setAttribute('data-id', i);
                    tabButtonEl.className = 'tab' + ((i == 1) ? ' selected' : '');
                    tabButtonEl.innerHTML = log;
                    tabButtonEl.addEventListener('click', tabClick);
                    logEl.append(tabButtonEl);
                    i++;
                });
                // Create div for log-content (table)
                let logContentEl = document.createElement('div');
                logContentEl.id = 'log-content';
                // Add table
                let logTbl = document.createElement('table');
                logTbl.id = 'log-table';
                let logTblHdr = document.createElement('thead');
                let logTableHdrStr = ['Timestamp', 'Level', 'Message'];
                logTableHdrStr.forEach(function (title) {
                    logTblHdr.appendChild(document.createElement("th")).appendChild(document.createTextNode(title));
                });
                logTbl.appendChild(logTblHdr);
                let logBodyEl = document.createElement('tbody');
                logBodyEl.id = 'log-table-rows';
                logTbl.appendChild(logBodyEl);
                logContentEl.appendChild(logTbl);
                logEl.append(logContentEl);
                // Request log for first tab
                if (i > 1) {
                    currentTabId = 1;
                    socket.send(JSON.stringify({ command: 'subscribeLog', logId: currentTabId,  maxLines: (showEmptyLines ? maxLines : maxLines * 2) }));
                }
            }
        }

        else if (response.event == 'logLines') {
            let logContentEl = document.getElementById('log-table-rows');
            // Add lines to table
            response.lines.forEach(function (logLine) {
                if (!showEmptyLines && logLine.msg == '') {
                    return;
                }
                let logTblRow = logContentEl.insertRow(0);
                logTblRow.className = 'level-' + logLine.level;
                if (!firstLogs) {
                    logTblRow.classList.add('new-log');
                }
                // Add cells
                let logTblCell = logTblRow.insertCell();
                logTblCell.appendChild(createSpanElementWithText(logLine.timestamp));
                logTblCell = logTblRow.insertCell();
                logTblCell.appendChild(createSpanElementWithText(logLine.level));
                logTblCell = logTblRow.insertCell();
                logTblCell.appendChild(createSpanElementWithText(htmlEncode(logLine.msg)));
                while (logContentEl.rows.length > maxLines) {
                    logContentEl.deleteRow(-1);
                }
            });
            firstLogs = false;
        }

        else if (response.event == 'error') {
            showError('Error received', response.message, 5000);
        }

        else {
            showError('Message error', 'Invalid event received:' + response.event, 5000);
        }
    };

    socket.onclose = function (event) {
        if (event.wasClean) {
            showError('WebSocket closed cleanly', 'code=${event.code} reason=${event.reason}', 0);
        } else {
            showError('WebSocket closed', 'Please reload page', 0);
        }
    };

    socket.onerror = function (error) {
        showError('WebSocket error', error.message, 5000);
    };

});